@@ -25,7 +25,7 @@
 
 #include <stdio.h>
 #include <limits.h>
-#include <math.h>
+#include "muse_math.h"
 
 #include <QComboBox>
 #include <QGridLayout>
@@ -77,14 +77,9 @@
 #include "shortcuts.h"
 #include "ttoolbutton.h"
 
-#ifdef _USE_TRACKINFO_ALT
-#include "mtrackinfo.h"
-#endif
-
 namespace MusEGui {
 
-std::vector<Arranger::custom_col_t> Arranger::custom_columns;     //FINDMICH TODO: eliminate all usage of new_custom_columns
-std::vector<Arranger::custom_col_t> Arranger::new_custom_columns; //and instead let the arranger update without restarting muse!
+std::vector<Arranger::custom_col_t> Arranger::custom_columns;
 QByteArray Arranger::header_state;
 static const char* gArrangerRasterStrings[] = {
       QT_TRANSLATE_NOOP("MusEGui::Arranger", "Off"), QT_TRANSLATE_NOOP("MusEGui::Arranger", "Bar"), "1/2", "1/4", "1/8", "1/16"
@@ -95,12 +90,12 @@ void Arranger::writeCustomColumns(int level, MusECore::Xml& xml)
 {
   xml.tag(level++, "custom_columns");
   
-  for (unsigned i=0;i<new_custom_columns.size();i++)
+  for (unsigned i = 0; i < custom_columns.size(); i++)
   {
     xml.tag(level++, "column");
-    xml.strTag(level, "name", new_custom_columns[i].name);
-    xml.intTag(level, "ctrl", new_custom_columns[i].ctrl);
-    xml.intTag(level, "affected_pos", new_custom_columns[i].affected_pos);
+    xml.strTag(level, "name", custom_columns[i].name);
+    xml.intTag(level, "ctrl", custom_columns[i].ctrl);
+    xml.intTag(level, "affected_pos", custom_columns[i].affected_pos);
     xml.etag(--level, "column");
   }
   
@@ -117,7 +112,6 @@ void Arranger::readCustomColumns(MusECore::Xml& xml)
             switch (token) {
                   case MusECore::Xml::Error:
                   case MusECore::Xml::End:
-                        new_custom_columns=custom_columns;
                         return;
                   case MusECore::Xml::TagStart:
                         if (tag == "column")
@@ -128,7 +122,6 @@ void Arranger::readCustomColumns(MusECore::Xml& xml)
                   case MusECore::Xml::TagEnd:
                         if (tag == "custom_columns")
                         {
-                              new_custom_columns=custom_columns;
                               return;
                         }
                   default:
@@ -174,15 +167,16 @@ Arranger::custom_col_t Arranger::readOneCustomColumn(MusECore::Xml& xml)
 
 void Arranger::setHeaderToolTips()
       {
+      header->setToolTip(COL_TRACK_IDX,  tr("Track index"));
       header->setToolTip(COL_INPUT_MONITOR, tr("Enable input monitor"));
-      header->setToolTip(COL_RECORD,     tr("Enable Recording"));
-      header->setToolTip(COL_MUTE,       tr("Mute/Off Indicator"));
-      header->setToolTip(COL_SOLO,       tr("Solo Indicator"));
-      header->setToolTip(COL_CLASS,      tr("Track Type"));
-      header->setToolTip(COL_NAME,       tr("Track Name"));
+      header->setToolTip(COL_RECORD,     tr("Enable recording"));
+      header->setToolTip(COL_MUTE,       tr("Mute/Off indicator"));
+      header->setToolTip(COL_SOLO,       tr("Solo indicator"));
+      header->setToolTip(COL_CLASS,      tr("Track type"));
+      header->setToolTip(COL_NAME,       tr("Track name"));
       header->setToolTip(COL_OCHANNEL,   tr("Midi output channel number or audio channels"));
       header->setToolTip(COL_OPORT,      tr("Midi output port or synth midi port"));
-      header->setToolTip(COL_TIMELOCK,   tr("Time Lock"));
+      header->setToolTip(COL_TIMELOCK,   tr("Time lock"));
       header->setToolTip(COL_AUTOMATION, tr("Automation parameter selection"));
       header->setToolTip(COL_CLEF,       tr("Notation clef"));
       }
@@ -195,6 +189,7 @@ void Arranger::setHeaderToolTips()
 
 void Arranger::setHeaderWhatsThis()
       {
+      header->setWhatsThis(COL_TRACK_IDX, tr("Track index"));
       header->setWhatsThis(COL_INPUT_MONITOR, tr("Enable input monitor. Click to toggle.\nPasses input through to output for monitoring.\n"
                                                  "See also Settings: Automatically Monitor On Record Arm."));
       header->setWhatsThis(COL_RECORD,   tr("Enable recording. Click to toggle.\n"
@@ -203,10 +198,10 @@ void Arranger::setHeaderWhatsThis()
       header->setWhatsThis(COL_SOLO,     tr("Solo indicator. Click to toggle.\nConnected tracks are also 'phantom' soloed,\n indicated by a dark square."));
       header->setWhatsThis(COL_CLASS,    tr("Track type. Right-click to change\n midi and drum track types."));
       header->setWhatsThis(COL_NAME,     tr("Track name. Double-click to edit.\nRight-click for more options."));
-      header->setWhatsThis(COL_OCHANNEL, tr("Midi/drum track: Output channel number.\nAudio track: Channels.\nMid/right-click to change."));
-      header->setWhatsThis(COL_OPORT,    tr("Midi/drum track: Output port.\nSynth track: Assigned midi port.\nLeft-click to change.\nRight-click to show GUI."));
+      header->setWhatsThis(COL_OCHANNEL, tr("Midi/Drum track: Output channel number.\nAudio track: Channels.\nMid/right-click to change."));
+      header->setWhatsThis(COL_OPORT,    tr("Midi/Drum track: Output port.\nSynth track: Assigned midi port.\nLeft-click to change.\nRight-click to show GUI."));
       header->setWhatsThis(COL_TIMELOCK, tr("Time lock"));
-      header->setToolTip(COL_CLEF,       tr("Notation clef. Select this tracks notation clef."));
+      header->setWhatsThis(COL_CLEF,     tr("Notation clef. Select this tracks notation clef."));
       }
 
 //---------------------------------------------------------
@@ -221,9 +216,6 @@ Arranger::Arranger(ArrangerView* parent, const char* name)
       _raster  = 0;      // measure
       selected = 0;
       showTrackinfoFlag = true;
-#ifdef _USE_TRACKINFO_ALT
-      showTrackinfoAltFlag = false;
-#endif
       
       cursVal = INT_MAX;
       
@@ -253,7 +245,6 @@ Arranger::Arranger(ArrangerView* parent, const char* name)
       toolbar->addWidget(label);
       cursorPos = new PosLabel(0);
       cursorPos->setEnabled(false);
-      cursorPos->setFixedHeight(22);
       toolbar->addWidget(cursorPos);
 
       label = new QLabel(tr("Snap"));
@@ -281,8 +272,8 @@ Arranger::Arranger(ArrangerView* parent, const char* name)
       lenEntry = new SpinBox(1, 10000, 1);
       lenEntry->setFocusPolicy(Qt::StrongFocus);
       lenEntry->setValue(MusEGlobal::song->len());
-      lenEntry->setToolTip(tr("song length - bars"));
-      lenEntry->setWhatsThis(tr("song length - bars"));
+      lenEntry->setToolTip(tr("Song length - bars"));
+      lenEntry->setWhatsThis(tr("Song length - bars"));
       toolbar->addWidget(lenEntry);
       connect(lenEntry, SIGNAL(valueChanged(int)), SLOT(songlenChanged(int)));
 
@@ -294,8 +285,8 @@ Arranger::Arranger(ArrangerView* parent, const char* name)
       globalPitchSpinBox = new SpinBox(-127, 127, 1);
       globalPitchSpinBox->setFocusPolicy(Qt::StrongFocus);
       globalPitchSpinBox->setValue(MusEGlobal::song->globalPitchShift());
-      globalPitchSpinBox->setToolTip(tr("midi pitch"));
-      globalPitchSpinBox->setWhatsThis(tr("global midi pitch shift"));
+      globalPitchSpinBox->setToolTip(tr("Midi pitch"));
+      globalPitchSpinBox->setWhatsThis(tr("Global midi pitch shift"));
       toolbar->addWidget(globalPitchSpinBox);
       connect(globalPitchSpinBox, SIGNAL(valueChanged(int)), SLOT(globalPitchChanged(int)));
       
@@ -308,8 +299,8 @@ Arranger::Arranger(ArrangerView* parent, const char* name)
       globalTempoSpinBox->setFocusPolicy(Qt::StrongFocus);
       globalTempoSpinBox->setSuffix(QString("%"));
       globalTempoSpinBox->setValue(MusEGlobal::tempomap.globalTempo());
-      globalTempoSpinBox->setToolTip(tr("midi tempo"));
-      globalTempoSpinBox->setWhatsThis(tr("midi tempo"));
+      globalTempoSpinBox->setToolTip(tr("Midi tempo"));
+      globalTempoSpinBox->setWhatsThis(tr("Midi tempo"));
       toolbar->addWidget(globalTempoSpinBox);
       connect(globalTempoSpinBox, SIGNAL(valueChanged(int)), SLOT(globalTempoChanged(int)));
       
@@ -375,55 +366,41 @@ Arranger::Arranger(ArrangerView* parent, const char* name)
 
       trackInfoButton  = new CompactToolButton(this);
       trackInfoButton->setContentsMargins(0, 0, 0, 0);
-      trackInfoButton->setText(tr("TrackInfo"));
+      trackInfoButton->setIcon(*mixerstripSVGIcon);
+      trackInfoButton->setToolTip(tr("Show mixer strip for current track (F5)"));
       trackInfoButton->setCheckable(true);
       trackInfoButton->setChecked(showTrackinfoFlag);
       trackInfoButton->setFocusPolicy(Qt::NoFocus);
-      trackInfoButton->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum));
+      trackInfoButton->setFixedSize(20, 14);
+      trackInfoButton->setDrawFlat(true);
+      trackInfoButton->setShortcut(QKeySequence(Qt::Key_F5));
+      trackInfoButton->setCursor(QCursor(Qt::PointingHandCursor));
       connect(trackInfoButton, SIGNAL(toggled(bool)), SLOT(showTrackInfo(bool)));
 
-#ifdef _USE_TRACKINFO_ALT
-      trackInfoAltButton  = new CompactToolButton(this);
-      trackInfoAltButton->setContentsMargins(0, 0, 0, 0);
-      trackInfoAltButton->setText(tr("Alt"));
-      trackInfoAltButton->setCheckable(true);
-      trackInfoAltButton->setChecked(showTrackinfoAltFlag);
-      trackInfoAltButton->setFocusPolicy(Qt::NoFocus);
-      trackInfoAltButton->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum));
-      connect(trackInfoAltButton, SIGNAL(toggled(bool)), SLOT(showTrackInfoAlt(bool)));
-#endif
-      
       genTrackInfo(trackInfoWidget);
 
       // set up the header
       header = new Header(tracklist, "header");
       header->setFixedHeight(31);
 
-      QFontMetrics fm1 = header->fontMetrics();
-      int fw = 11;
-
-      header->setColumnLabel(tr("I"), COL_INPUT_MONITOR);
-      header->setColumnLabel(tr("R"), COL_RECORD);
-      header->setColumnLabel(tr("M"), COL_MUTE);
-      header->setColumnLabel(tr("S"), COL_SOLO);
-      header->setColumnLabel(tr("C"), COL_CLASS);
-      header->setColumnLabel(tr("Track"), COL_NAME, 100);
-      header->setColumnLabel(tr("Port"), COL_OPORT, 60);
+      header->setColumnLabel(tr("#"), COL_TRACK_IDX);
+      header->setColumnIcon(*monitorOnSVGIcon, COL_INPUT_MONITOR);
+      header->setColumnIcon(*recArmOnSVGIcon, COL_RECORD);
+      header->setColumnIcon(*muteOnSVGIcon, COL_MUTE);
+      header->setColumnIcon(*soloOnAloneSVGIcon, COL_SOLO);
+      header->setColumnIcon(*tracktypeSVGIcon, COL_CLASS);
+      header->setColumnLabel(tr("Track"), COL_NAME);
+      header->setColumnLabel(tr("Port"), COL_OPORT);
+      //: Channel
       header->setColumnLabel(tr("Ch"), COL_OCHANNEL);
+      //: Time lock
       header->setColumnLabel(tr("T"), COL_TIMELOCK);
-      header->setColumnLabel(tr("Automation"), COL_AUTOMATION, 75);
-      header->setColumnLabel(tr("Clef"), COL_CLEF, 75);
-      for (unsigned i=0;i<custom_columns.size();i++)
-        header->setColumnLabel(custom_columns[i].name, COL_CUSTOM_MIDICTRL_OFFSET+i, MAX(fm1.width(custom_columns[i].name)+fw, 30));
-
-      header->resizeSection(COL_INPUT_MONITOR, fm1.width(header->columnLabel(COL_INPUT_MONITOR)) + fw);
-      header->resizeSection(COL_RECORD, fm1.width(header->columnLabel(COL_RECORD)) + fw);
-      header->resizeSection(COL_MUTE, fm1.width(header->columnLabel(COL_MUTE)) + fw);
-      header->resizeSection(COL_SOLO, fm1.width(header->columnLabel(COL_SOLO)) + fw);
-      header->resizeSection(COL_CLASS, fm1.width(header->columnLabel(COL_CLASS)) + fw);
-      header->resizeSection(COL_OCHANNEL, fm1.width(header->columnLabel(COL_OCHANNEL)) + fw);
-      header->resizeSection(COL_TIMELOCK, fm1.width(header->columnLabel(COL_TIMELOCK)) + fw);
+      header->setColumnLabel(tr("Automation"), COL_AUTOMATION);
+      header->setColumnLabel(tr("Clef"), COL_CLEF);
+      for (unsigned i = 0; i < custom_columns.size(); i++)
+         header->setColumnLabel(custom_columns[i].name, COL_CUSTOM_MIDICTRL_OFFSET + i);
 
+      header->setSectionResizeMode(COL_TRACK_IDX, QHeaderView::Interactive);
       header->setSectionResizeMode(COL_INPUT_MONITOR, QHeaderView::Fixed);
       header->setSectionResizeMode(COL_RECORD, QHeaderView::Fixed);
       header->setSectionResizeMode(COL_MUTE, QHeaderView::Fixed);
@@ -435,7 +412,7 @@ Arranger::Arranger(ArrangerView* parent, const char* name)
       header->setSectionResizeMode(COL_TIMELOCK, QHeaderView::Fixed);
       header->setSectionResizeMode(COL_AUTOMATION, QHeaderView::Interactive);
       header->setSectionResizeMode(COL_CLEF, QHeaderView::Interactive);
-      for (unsigned i=0;i<custom_columns.size();i++)
+      for (unsigned i = 0; i < custom_columns.size(); i++)
         header->setSectionResizeMode(COL_CUSTOM_MIDICTRL_OFFSET+i, QHeaderView::Interactive);
 
       // 04/18/17 Time lock remains unused. Disabled until a use is found.
@@ -475,17 +452,12 @@ Arranger::Arranger(ArrangerView* parent, const char* name)
       //    Editor
       //---------------------------------------------------
 
-      int offset = AL::sigmap.ticksMeasure(0);
+      int offset = MusEGlobal::sigmap.ticksMeasure(0);
       hscroll = new ScrollScale(-2000, -5, xscale, MusEGlobal::song->len(), Qt::Horizontal, this, -offset);
       hscroll->setFocusPolicy(Qt::NoFocus);
       hscroll->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
       
-#ifdef _USE_TRACKINFO_ALT
-      bottomHLayout = new ArrangerHScrollLayout(0, trackInfoButton, trackInfoAltButton, hscroll, editor);
-#else
-      // No Alt panel here.
       bottomHLayout = new ArrangerHScrollLayout(0, trackInfoButton, 0, hscroll, editor);
-#endif
       
       box->addLayout(bottomHLayout);
       bottomHLayout->setContentsMargins(0, 0, 0, 0);
@@ -535,16 +507,9 @@ Arranger::Arranger(ArrangerView* parent, const char* name)
       connect(globalPitchSpinBox, SIGNAL(escapePressed()), SLOT(focusCanvas()));
       connect(globalTempoSpinBox, SIGNAL(returnPressed()), SLOT(focusCanvas()));
       connect(globalTempoSpinBox, SIGNAL(escapePressed()), SLOT(focusCanvas()));
-#ifdef _USE_TRACKINFO_ALT
-      connect(midiTrackInfo,      SIGNAL(returnPressed()), SLOT(focusCanvas()));
-      connect(midiTrackInfo,      SIGNAL(escapePressed()), SLOT(focusCanvas()));
-#endif
 
       //connect(this,      SIGNAL(redirectWheelEvent(QWheelEvent*)), canvas, SLOT(redirectedWheelEvent(QWheelEvent*)));
       connect(list,      SIGNAL(redirectWheelEvent(QWheelEvent*)), canvas, SLOT(redirectedWheelEvent(QWheelEvent*)));
-#ifdef _USE_TRACKINFO_ALT
-      //connect(trackInfo, SIGNAL(redirectWheelEvent(QWheelEvent*)), infoScroll, SLOT(redirectedWheelEvent(QWheelEvent*)));
-#endif
       
       egrid->addWidget(time, 0, 0, 1, 2);
       egrid->addWidget(MusECore::hLine(editor), 1, 0, 1, 2);
@@ -570,7 +535,7 @@ Arranger::Arranger(ArrangerView* parent, const char* name)
       connect(canvas, SIGNAL(dclickPart(MusECore::Track*)), SIGNAL(editPart(MusECore::Track*)));
       connect(canvas, SIGNAL(startEditor(MusECore::PartList*,int)),   SIGNAL(startEditor(MusECore::PartList*, int)));
 
-      connect(MusEGlobal::song,   SIGNAL(songChanged(MusECore::SongChangedFlags_t)), SLOT(songChanged(MusECore::SongChangedFlags_t)));
+      connect(MusEGlobal::song,   SIGNAL(songChanged(MusECore::SongChangedStruct_t)), SLOT(songChanged(MusECore::SongChangedStruct_t)));
       connect(canvas, SIGNAL(followEvent(int)), hscroll, SLOT(setOffset(int)));
 
       connect(canvas, SIGNAL(dropSongFile(const QString&)), SIGNAL(dropSongFile(const QString&)));
@@ -580,21 +545,12 @@ Arranger::Arranger(ArrangerView* parent, const char* name)
       connect(MusEGlobal::song,   SIGNAL(controllerChanged(MusECore::Track*, int)), SLOT(controllerChanged(MusECore::Track*, int)));
 
       configChanged();  // set configuration values
-#ifdef _USE_TRACKINFO_ALT
-      if(canvas->part())
-        midiTrackInfo->setTrack(canvas->part()->track());   
-#endif
       showTrackInfo(showTrackinfoFlag);
       
       setDefaultSplitterSizes();
       
       // Take care of some tabbies!
-//       setTabOrder(tempo200, trackInfo);
-//       setTabOrder(trackInfo, infoScroll);
-//       setTabOrder(infoScroll, list);
       setTabOrder(list, canvas);
-      //setTabOrder(canvas, ib);
-      //setTabOrder(ib, hscroll);
       }
 
 //---------------------------------------------------------
@@ -605,7 +561,8 @@ void Arranger::setDefaultSplitterSizes()
 {
   QList<int> vallist;
   vallist.append(trackInfoWidget->minimumSize().width());
-  vallist.append(tlistLayout->sizeHint().width());
+//  vallist.append(tlistLayout->sizeHint().width());
+  vallist.append(250);
   vallist.append(300);
   split->setSizes(vallist);
 }
@@ -650,17 +607,25 @@ void Arranger::dclickPart(MusECore::Track* t)
 
 void Arranger::setHeaderSizes()
 {
-  QFontMetrics fm1(header->font());
-  int fw = 11;
-  header->resizeSection(COL_INPUT_MONITOR, fm1.width(header->columnLabel(COL_INPUT_MONITOR)) + fw);
-  header->resizeSection(COL_RECORD, fm1.width(header->columnLabel(COL_RECORD)) + fw);
-  header->resizeSection(COL_MUTE, fm1.width(header->columnLabel(COL_MUTE)) + fw);
-  header->resizeSection(COL_SOLO, fm1.width(header->columnLabel(COL_SOLO)) + fw);
-  header->resizeSection(COL_CLASS, fm1.width(header->columnLabel(COL_CLASS)) + fw);
-  header->resizeSection(COL_OCHANNEL, fm1.width(header->columnLabel(COL_OCHANNEL)) + fw);
-  header->resizeSection(COL_TIMELOCK, fm1.width(header->columnLabel(COL_TIMELOCK)) + fw);
-  for (unsigned i=0;i<custom_columns.size();i++)
-    header->resizeSection(COL_CUSTOM_MIDICTRL_OFFSET+i, MAX(fm1.width(custom_columns[i].name)+fw, 30));
+    const int fw = 11;
+
+    header->resizeSection(COL_TRACK_IDX, qMax(header->sectionSizeHint(COL_TRACK_IDX) + fw, 30));
+
+    header->resizeSection(COL_INPUT_MONITOR, header->sectionSizeHint(COL_INPUT_MONITOR));
+    header->resizeSection(COL_RECORD, header->sectionSizeHint(COL_RECORD));
+    header->resizeSection(COL_MUTE, header->sectionSizeHint(COL_MUTE));
+    header->resizeSection(COL_SOLO, header->sectionSizeHint(COL_SOLO));
+    header->resizeSection(COL_CLASS, header->sectionSizeHint(COL_CLASS));
+
+    header->resizeSection(COL_NAME, qMax(header->sectionSizeHint(COL_NAME) + fw, 100));
+    header->resizeSection(COL_OPORT, qMax(header->sectionSizeHint(COL_OPORT) + fw, 60));
+    header->resizeSection(COL_OCHANNEL, header->sectionSizeHint(COL_OCHANNEL) + fw);
+    header->resizeSection(COL_TIMELOCK, header->sectionSizeHint(COL_TIMELOCK) + fw);
+    header->resizeSection(COL_AUTOMATION, qMax(header->sectionSizeHint(COL_AUTOMATION) + fw, 80));
+    header->resizeSection(COL_CLEF, qMax(header->sectionSizeHint(COL_CLEF) + fw, 50));
+
+    for (unsigned i = 0; i < custom_columns.size(); i++)
+        header->resizeSection(COL_CUSTOM_MIDICTRL_OFFSET + i, qMax(header->sectionSizeHint(COL_CUSTOM_MIDICTRL_OFFSET + i) + fw, 30));
 }
 
 //---------------------------------------------------------
@@ -677,6 +642,7 @@ void Arranger::configChanged()
             canvas->setBg(QPixmap(MusEGlobal::config.canvasBgPixmap));
       }
       setHeaderSizes();
+      _parentWin->updateVisibleTracksButtons();
       }
 
 //---------------------------------------------------------
@@ -698,7 +664,7 @@ void Arranger::focusCanvas()
 
 void Arranger::songlenChanged(int n)
       {
-      int newLen = AL::sigmap.bar2tick(n, 0, 0);
+      int newLen = MusEGlobal::sigmap.bar2tick(n, 0, 0);
       MusEGlobal::song->setLen(newLen);
       }
       
@@ -706,49 +672,8 @@ void Arranger::songlenChanged(int n)
 //   songChanged
 //---------------------------------------------------------
 
-void Arranger::songChanged(MusECore::SongChangedFlags_t type)
+void Arranger::songChanged(MusECore::SongChangedStruct_t type)
       {
-#ifdef _USE_TRACKINFO_ALT
-        // We must catch this first and be sure to update the strips.
-        if(type & SC_TRACK_REMOVED)
-        {
-          {
-            AudioStrip* w = static_cast<AudioStrip*>(trackInfoWidget->getWidget(2));
-            if(w)
-            {
-              MusECore::Track* t = w->getTrack();
-              if(t)
-              {
-                if(!MusEGlobal::song->trackExists(t))
-                {
-                  delete w;
-                  trackInfoWidget->addWidget(0, 2);
-                  selected = 0;
-                  switchInfo(0);
-                } 
-              }   
-            } 
-          }
-          
-          {
-            MidiStrip* w = static_cast<MidiStrip*>(trackInfoWidget->getWidget(3));
-            if(w)
-            {
-              MusECore::Track* t = w->getTrack();
-              if(t)
-              {
-                if(!MusEGlobal::song->trackExists(t))
-                {
-                  delete w;
-                  trackInfoWidget->addWidget(0, 3);
-                  selected = 0;
-                  switchInfo(0);
-                } 
-              }   
-            } 
-          }
-        }
-#else
         // We must catch this first and be sure to update the strips.
         if(type & SC_TRACK_REMOVED)
         {
@@ -788,7 +713,6 @@ void Arranger::songChanged(MusECore::SongChangedFlags_t type)
             } 
           }
         }
-#endif
         
         // Try these, may need more/less. 
         if(type & ( SC_TRACK_INSERTED | SC_TRACK_REMOVED | SC_TRACK_MODIFIED | 
@@ -796,14 +720,14 @@ void Arranger::songChanged(MusECore::SongChangedFlags_t type)
            SC_PART_INSERTED | SC_PART_REMOVED | SC_PART_MODIFIED))  
         {
           unsigned endTick = MusEGlobal::song->len();
-          int offset  = AL::sigmap.ticksMeasure(endTick);
+          int offset  = MusEGlobal::sigmap.ticksMeasure(endTick);
           hscroll->setRange(-offset, endTick + offset);  //DEBUG
           canvas->setOrigin(-offset, 0);
           time->setOrigin(-offset, 0);
     
           int bar, beat;
           unsigned tick;
-          AL::sigmap.tickValues(endTick, &bar, &beat, &tick);
+          MusEGlobal::sigmap.tickValues(endTick, &bar, &beat, &tick);
           if (tick || beat)
                 ++bar;
           lenEntry->blockSignals(true);
@@ -821,7 +745,14 @@ void Arranger::songChanged(MusECore::SongChangedFlags_t type)
                    SC_TRACK_MOVED | SC_TRACK_RESIZED |
                    SC_PART_INSERTED | SC_PART_REMOVED | SC_PART_MODIFIED | 
                    SC_SIG | SC_TEMPO | SC_MASTER)) 
-          canvas->partsChanged();
+          canvas->updateItems();
+        
+        if(type & (SC_PART_SELECTION))
+        {
+          // Prevent race condition: Ignore if the change was ultimately sent by the canvas itself.
+          if(type._sender != canvas)
+            canvas->updateItemSelections();
+        }
         
         if (type & SC_SIG)
               time->redraw();
@@ -831,7 +762,7 @@ void Arranger::songChanged(MusECore::SongChangedFlags_t type)
         // Try these:
         if(type & (SC_PART_INSERTED | SC_PART_REMOVED | SC_PART_MODIFIED |
                    SC_EVENT_INSERTED | SC_EVENT_REMOVED | SC_EVENT_MODIFIED |
-                   SC_CLIP_MODIFIED))
+                   SC_CLIP_MODIFIED | SC_MARKER_INSERTED | SC_MARKER_REMOVED | SC_MARKER_MODIFIED))
         canvas->redraw();
         
         // We must marshall song changed instead of connecting to the strip's song changed
@@ -842,6 +773,14 @@ void Arranger::songChanged(MusECore::SongChangedFlags_t type)
         //  the strips each time no matter what the flags are !
         //updateTrackInfo(type);
         trackInfoSongChange(type);
+
+        // Update the arrangerview's actions.
+        // This needs to come after the canvas->selectionChanged() above so that in
+        //  selectionChanged(), itemsAreSelected() has the latest citems' selected flags.
+        if(type & (SC_TRACK_SELECTION | SC_PART_SELECTION | 
+                  SC_TRACK_INSERTED | SC_TRACK_REMOVED | SC_TRACK_MODIFIED | 
+                  SC_PART_INSERTED | SC_PART_REMOVED | SC_PART_MODIFIED))
+          _parentWin->selectionChanged();
     }
 
 //---------------------------------------------------------
@@ -1070,7 +1009,7 @@ void Arranger::globalPitchChanged(int val)
 
 void Arranger::globalTempoChanged(int val)
       {
-      MusEGlobal::audio->msgSetGlobalTempo(val);
+      MusEGlobal::song->applyOperation(MusECore::UndoOp(MusECore::UndoOp::SetGlobalTempo, val, 0));
       }
 
 //---------------------------------------------------------
@@ -1079,7 +1018,7 @@ void Arranger::globalTempoChanged(int val)
 
 void Arranger::setTempo50()
       {
-      MusEGlobal::audio->msgSetGlobalTempo(50);
+      MusEGlobal::song->applyOperation(MusECore::UndoOp(MusECore::UndoOp::SetGlobalTempo, 50, 0));
       }
 
 //---------------------------------------------------------
@@ -1088,7 +1027,7 @@ void Arranger::setTempo50()
 
 void Arranger::setTempo100()
       {
-      MusEGlobal::audio->msgSetGlobalTempo(100);
+      MusEGlobal::song->applyOperation(MusECore::UndoOp(MusECore::UndoOp::SetGlobalTempo, 100, 0));
       }
 
 //---------------------------------------------------------
@@ -1097,7 +1036,7 @@ void Arranger::setTempo100()
 
 void Arranger::setTempo200()
       {
-      MusEGlobal::audio->msgSetGlobalTempo(200);
+      MusEGlobal::song->applyOperation(MusECore::UndoOp(MusECore::UndoOp::SetGlobalTempo, 200, 0));
       }
 
 //---------------------------------------------------------
@@ -1128,24 +1067,6 @@ void Arranger::verticalScrollSetYpos(unsigned ypos)
 
 void Arranger::clear()
       {
-#ifdef _USE_TRACKINFO_ALT
-      {
-        AudioStrip* w = static_cast<AudioStrip*>(trackInfoWidget->getWidget(2));
-        if (w)
-              delete w;
-        trackInfoWidget->addWidget(0, 2);
-      }
-      
-      {
-        MidiStrip* w = static_cast<MidiStrip*>(trackInfoWidget->getWidget(3));
-        if (w)
-              delete w;
-        trackInfoWidget->addWidget(0, 3);
-      }
-      
-      selected = 0;
-      midiTrackInfo->setTrack(0);
-#else
       {
         AudioStrip* w = static_cast<AudioStrip*>(trackInfoWidget->getWidget(1));
         if (w)
@@ -1161,7 +1082,6 @@ void Arranger::clear()
       }
       
       selected = 0;
-#endif
       }
 
 //void Arranger::wheelEvent(QWheelEvent* ev)
@@ -1180,23 +1100,12 @@ void Arranger::controllerChanged(MusECore::Track *t, int ctrlId)
 
 void Arranger::showTrackInfo(bool flag)
       {
+      trackInfoButton->setToolTip(flag ? tr("Hide mixer strip for current track (F5)") : tr("Show mixer strip for current track (F5)"));
       showTrackinfoFlag = flag;
       trackInfoWidget->setVisible(flag);
       updateTrackInfo(-1);
       }
 
-#ifdef _USE_TRACKINFO_ALT
-//---------------------------------------------------------
-//   showTrackInfoAlt
-//---------------------------------------------------------
-
-void Arranger::showTrackInfoAlt(bool flag)
-      {
-      showTrackinfoAltFlag = flag;
-      updateTrackInfo(-1);
-      }
-#endif
-
 //---------------------------------------------------------
 //   genTrackInfo
 //---------------------------------------------------------
@@ -1217,25 +1126,16 @@ void Arranger::genTrackInfo(TrackInfoWidget* trackInfo)
       noTrackInfo->setGeometry(0, 0, 65, 200);
       noTrackInfo->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Expanding));
 
-#ifdef _USE_TRACKINFO_ALT
-      midiTrackInfo = new MidiTrackInfo(trackInfo);
-      
-      trackInfo->addWidget(noTrackInfo, 0);
-      trackInfo->addWidget(midiTrackInfo, 1);
-      trackInfo->addWidget(0, 2);  // AudioStrip placeholder.
-      trackInfo->addWidget(0, 3);  // MidiStrip placeholder.
-#else
       trackInfo->addWidget(noTrackInfo, 0);
       trackInfo->addWidget(0, 1);  // AudioStrip placeholder.
       trackInfo->addWidget(0, 2);  // MidiStrip placeholder.
-#endif
       }
 
 //---------------------------------------------------------
 //   updateTrackInfo
 //---------------------------------------------------------
 
-void Arranger::updateTrackInfo(MusECore::SongChangedFlags_t /*flags*/)
+void Arranger::updateTrackInfo(MusECore::SongChangedStruct_t /*flags*/)
       {
       if (!showTrackinfoFlag) {
             switchInfo(-1);
@@ -1246,30 +1146,10 @@ void Arranger::updateTrackInfo(MusECore::SongChangedFlags_t /*flags*/)
             return;
             }
       if (selected->isMidiTrack()) {
-#ifdef _USE_TRACKINFO_ALT
-            if(showTrackinfoAltFlag)
-            {
-              switchInfo(1);
-              // If a different part was selected
-              if(midiTrackInfo->track() != selected)
-                // Set a new track and do a complete update.
-                midiTrackInfo->setTrack(selected);
-              else  
-                // Otherwise just regular update with specific flags.
-                midiTrackInfo->updateTrackInfo(flags);
-            }
-            else
-              switchInfo(3);
-#else
               switchInfo(2);
-#endif
             }
       else {
-#ifdef _USE_TRACKINFO_ALT
-            switchInfo(2);
-#else
             switchInfo(1);
-#endif
             }
       }
 
@@ -1277,102 +1157,6 @@ void Arranger::updateTrackInfo(MusECore::SongChangedFlags_t /*flags*/)
 //   switchInfo
 //---------------------------------------------------------
 
-#ifdef _USE_TRACKINFO_ALT
-void Arranger::switchInfo(int n)
-      {
-      if (n == 2) {
-          {
-            MidiStrip* w = static_cast<MidiStrip*>(trackInfoWidget->getWidget(3));
-            if (w)
-            {
-              //fprintf(stderr, "Arranger::switchInfo audio strip: deleting midi strip\n");
-              delete w;
-              //w->deleteLater();
-              trackInfoWidget->addWidget(0, 3);
-            }
-          }
-          {
-              AudioStrip* w = static_cast<AudioStrip*>(trackInfoWidget->getWidget(2));
-              if (w == 0 || selected != w->getTrack()) {
-                    if (w)
-                    {
-                          //fprintf(stderr, "Arranger::switchInfo deleting strip\n");
-                          delete w;
-                          //w->deleteLater();
-                    }
-                    w = new AudioStrip(trackInfoWidget, static_cast<MusECore::AudioTrack*>(selected));
-                    // Broadcast changes to other selected tracks.
-                    w->setBroadcastChanges(true);
-
-                    // Set focus yielding to the canvas.
-                    if(MusEGlobal::config.smartFocus)
-                    {
-                      w->setFocusYieldWidget(canvas);
-                      //w->setFocusPolicy(Qt::WheelFocus);
-                    }
-
-                    // We must marshall song changed instead of connecting to the strip's song changed
-                    //  otherwise it crashes when loading another song because track is no longer valid
-                    //  and the strip's songChanged() seems to be called before Arranger songChanged()
-                    //  gets called and has a chance to stop the crash.
-                    //connect(MusEGlobal::song, SIGNAL(songChanged(MusECore::SongChangedFlags_t)), w, SLOT(songChanged(MusECore::SongChangedFlags_t)));
-                    
-                    connect(MusEGlobal::muse, SIGNAL(configChanged()), w, SLOT(configChanged()));
-                    w->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed));
-                    trackInfoWidget->addWidget(w, 2);
-                    w->show();
-                    //setTabOrder(midiTrackInfo, w);
-                    }
-          }
-        }
-
-      else if (n == 3) {
-          {
-            AudioStrip* w = static_cast<AudioStrip*>(trackInfoWidget->getWidget(2));
-            if (w)
-            {
-              //fprintf(stderr, "Arranger::switchInfo midi strip: deleting audio strip\n");
-              delete w;
-              //w->deleteLater();
-              trackInfoWidget->addWidget(0, 2);
-            }
-          }
-          {
-            MidiStrip* w = static_cast<MidiStrip*>(trackInfoWidget->getWidget(3));
-            if (w == 0 || selected != w->getTrack()) {
-                  if (w)
-                  {
-                        //fprintf(stderr, "Arranger::switchInfo deleting strip\n");
-                        delete w;
-                        //w->deleteLater();
-                  }
-                  w = new MidiStrip(trackInfoWidget, static_cast<MusECore::MidiTrack*>(selected));
-                  // Broadcast changes to other selected tracks.
-                  w->setBroadcastChanges(true);
-                  // Set focus yielding to the arranger canvas.
-                  if(MusEGlobal::config.smartFocus)
-                  {
-                    w->setFocusYieldWidget(canvas);
-                    //w->setFocusPolicy(Qt::WheelFocus);
-                  }
-
-                  // No. See above.
-                  //connect(MusEGlobal::song, SIGNAL(songChanged(MusECore::SongChangedFlags_t)), w, SLOT(songChanged(MusECore::SongChangedFlags_t)));
-                  
-                  connect(MusEGlobal::muse, SIGNAL(configChanged()), w, SLOT(configChanged()));
-                  w->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed));
-                  trackInfoWidget->addWidget(w, 3);
-                  w->show();
-                  //setTabOrder(midiTrackInfo, w);
-                  }
-          }
-        }
-            
-      if (trackInfoWidget->curIdx() == n)
-            return;
-      trackInfoWidget->raiseWidget(n);
-      }
-#else
 void Arranger::switchInfo(int n)
       {
       if (n == 1) {
@@ -1410,7 +1194,7 @@ void Arranger::switchInfo(int n)
                     //  otherwise it crashes when loading another song because track is no longer valid
                     //  and the strip's songChanged() seems to be called before Arranger songChanged()
                     //  gets called and has a chance to stop the crash.
-                    //connect(MusEGlobal::song, SIGNAL(songChanged(MusECore::SongChangedFlags_t)), w, SLOT(songChanged(MusECore::SongChangedFlags_t)));
+                    //connect(MusEGlobal::song, SIGNAL(songChanged(MusECore::SongChangedStruct_t)), w, SLOT(songChanged(MusECore::SongChangedStruct_t)));
                     
                     connect(MusEGlobal::muse, SIGNAL(configChanged()), w, SLOT(configChanged()));
                     w->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed));
@@ -1452,7 +1236,7 @@ void Arranger::switchInfo(int n)
                   }
 
                   // No. See above.
-                  //connect(MusEGlobal::song, SIGNAL(songChanged(MusECore::SongChangedFlags_t)), w, SLOT(songChanged(MusECore::SongChangedFlags_t)));
+                  //connect(MusEGlobal::song, SIGNAL(songChanged(MusECore::SongChangedStruct_t)), w, SLOT(songChanged(MusECore::SongChangedStruct_t)));
                   
                   connect(MusEGlobal::muse, SIGNAL(configChanged()), w, SLOT(configChanged()));
                   w->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed));
@@ -1467,43 +1251,12 @@ void Arranger::switchInfo(int n)
             return;
       trackInfoWidget->raiseWidget(n);
       }
-#endif
 
 //---------------------------------------------------------
 //   trackInfoSongChange
 //---------------------------------------------------------
 
-#ifdef _USE_TRACKINFO_ALT
-void Arranger::trackInfoSongChange(MusECore::SongChangedFlags_t flags)
-{
-  if(!selected || !showTrackinfoFlag)
-    return;
-
-  // Only update what is showing.
-  if(selected->isMidiTrack()) 
-  {
-    if(showTrackinfoAltFlag)
-    {
-      MidiTrackInfo* w = static_cast<MidiTrackInfo*>(trackInfoWidget->getWidget(1));
-      if(w)
-        w->songChanged(flags);
-    }
-    else
-    {
-      MidiStrip* w = static_cast<MidiStrip*>(trackInfoWidget->getWidget(3));
-      if(w)
-        w->songChanged(flags);
-    }
-  }
-  else 
-  {
-    AudioStrip* w = static_cast<AudioStrip*>(trackInfoWidget->getWidget(2));
-    if(w)
-      w->songChanged(flags);
-  }
-}
-#else
-void Arranger::trackInfoSongChange(MusECore::SongChangedFlags_t flags)
+void Arranger::trackInfoSongChange(MusECore::SongChangedStruct_t flags)
 {
   if(!selected || !showTrackinfoFlag)
     return;
@@ -1522,7 +1275,6 @@ void Arranger::trackInfoSongChange(MusECore::SongChangedFlags_t flags)
       w->songChanged(flags);
   }
 }
-#endif
 
 void Arranger::keyPressEvent(QKeyEvent* event)
 {
@@ -1576,5 +1328,20 @@ void Arranger::horizontalZoom(int mag, const QPoint& glob_pos)
     hscroll->setMag(hscroll->mag() + mag, cp.x());
 }
 
+void Arranger::updateHeaderCustomColumns()
+{
+    for (int i = COL_CUSTOM_MIDICTRL_OFFSET; i < header->count(); i++)
+        header->removeColumn(i);
+
+    header->headerDataChanged(Qt::Horizontal, COL_CUSTOM_MIDICTRL_OFFSET, header->count());
+
+    for (unsigned i = 0; i < custom_columns.size(); i++) {
+        header->setColumnLabel(custom_columns[i].name, COL_CUSTOM_MIDICTRL_OFFSET + i);
+        header->showSection(COL_CUSTOM_MIDICTRL_OFFSET + i);
+    }
+
+    setHeaderSizes();
+    list->redraw();
+}
 
 } // namespace MusEGui
