@@ -3,6 +3,7 @@
 //  Linux Music Editor
 //    $Id: arranger.h,v 1.17.2.15 2009/11/14 03:37:48 terminator356 Exp $
 //  (C) Copyright 1999 Werner Schweer (ws@seh.de)
+//  (C) Copyright 2016 Tim E. Real (terminator356 on sourceforge)
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -25,25 +26,23 @@
 
 #include <vector>
 #include <QString>
-
+#include <QAction>
+#include <QCheckBox>
+#include <QMainWindow>
+#include <QMenu>
+#include <QWheelEvent>
+#include <QKeyEvent>
+#include <QPoint>
+#include <QComboBox>
 #include <QScrollBar>
-#include <QResizeEvent>
+#include <QVBoxLayout>
+#include <QHBoxLayout>
 
 #include "type_defs.h"
 #include "midieditor.h"
 #include "pcanvas.h"
 #include "trackautomationview.h"
 
-class QAction;
-class QCheckBox;
-class QMainWindow;
-class QMenu;
-class QToolButton;
-class QWheelEvent;
-class QKeyEvent;
-class QPoint;
-class QComboBox;
-
 namespace MusECore {
 class Track;
 class Xml;
@@ -55,58 +54,15 @@ class AudioStrip;
 class Header;
 class LabelCombo;
 class MTScale;
-class MidiTrackInfo;
 class PosLabel;
 class ScrollScale;
 class SpinBox;
 class Splitter;
-class TLLayout;
+class TrackInfoWidget;
 class TList;
-class WidgetStack;
-
-
-//---------------------------------------------------------
-//   WidgetStack
-//---------------------------------------------------------
-
-class WidgetStack : public QWidget {
-      Q_OBJECT
-      std::vector<QWidget*> stack;
-      int top;
-
-   protected:
-      virtual void wheelEvent(QWheelEvent* e);
-      
-   signals:
-      void redirectWheelEvent(QWheelEvent*);
-      
-   public:
-      WidgetStack(QWidget* parent, const char* name = 0);
-      void raiseWidget(int idx);
-      void addWidget(QWidget* w, unsigned int idx);
-      QWidget* getWidget(unsigned int idx);
-      QWidget* visibleWidget() const;
-      int curIdx() const { return top; }
-      virtual QSize minimumSizeHint() const;
-      };
-
-//---------------------------------------------------------
-//   ScrollBar
-//---------------------------------------------------------
-
-class ScrollBar : public QScrollBar {
-      Q_OBJECT
-      
-  public slots:
-      void redirectedWheelEvent(QWheelEvent*);
-      
-  protected:
-      virtual void resizeEvent(QResizeEvent* e) { setPageStep(e->size().height()); }
-      
-  public:    
-    ScrollBar(Qt::Orientation orientation, QWidget * parent = 0 ) : QScrollBar(orientation, parent) {};  
-};
-
+class ArrangerCanvasLayout;
+class ArrangerHScrollLayout;
+class CompactToolButton;
 
 //---------------------------------------------------------
 //   Arranger
@@ -124,22 +80,22 @@ class Arranger : public QWidget {
       PartCanvas* canvas;
       ScrollScale* hscroll;
       QScrollBar* vscroll;
+      QVBoxLayout* tlistLayout;
+      ArrangerCanvasLayout* egrid;
+      ArrangerHScrollLayout* bottomHLayout;
       TList* list;
       Header* header;
       MTScale* time;
       SpinBox* lenEntry;
       bool showTrackinfoFlag;
-      WidgetStack* trackInfo;
-      ScrollBar* infoScroll;
-      MidiTrackInfo* midiTrackInfo;
+      TrackInfoWidget* trackInfoWidget;
       AudioStrip* waveTrackInfo;
       QWidget* noTrackInfo;
       QWidget* tracklist;
-      TLLayout* tgrid;
 
       MusECore::Track* selected;
 
-      QToolButton* ib;
+      CompactToolButton* trackInfoButton;
       int trackInfoType;
       Splitter* split;
       int songType;
@@ -148,20 +104,21 @@ class Arranger : public QWidget {
       SpinBox* globalPitchSpinBox;
       
       unsigned cursVal;
-      void genTrackInfo(QWidget* parent);
+      void genTrackInfo(TrackInfoWidget*);
       void genMidiTrackInfo();
       void genWaveTrackInfo();
       void switchInfo(int);
+      void trackInfoSongChange(MusECore::SongChangedStruct_t flags);
       void setHeaderToolTips();
       void setHeaderWhatsThis();
+      void setHeaderSizes();
 
    private slots:
       void rasterChanged(int);
       void songlenChanged(int);
       void showTrackInfo(bool);
       void trackSelectionChanged();
-      void trackInfoScroll(int);
-      void songChanged(MusECore::SongChangedFlags_t);
+      void songChanged(MusECore::SongChangedStruct_t);
       void setTime(unsigned);
       void globalPitchChanged(int);
       void globalTempoChanged(int);
@@ -174,7 +131,6 @@ class Arranger : public QWidget {
       
    signals:
       void editPart(MusECore::Track*);
-      void selectionChanged();
       void dropSongFile(const QString&);
       void dropMidiFile(const QString&);
       void startEditor(MusECore::PartList*, int);
@@ -188,7 +144,7 @@ class Arranger : public QWidget {
    public slots:
       void dclickPart(MusECore::Track*);
       void setTool(int);
-      void updateTrackInfo(MusECore::SongChangedFlags_t flags);
+      void updateTrackInfo(MusECore::SongChangedStruct_t flags);
       void configChanged();
       void controllerChanged(MusECore::Track *t, int ctrlId);
       void focusCanvas();
@@ -212,8 +168,7 @@ class Arranger : public QWidget {
           affected_pos=a;
         }
       };
-      static std::vector<custom_col_t> custom_columns;     //FINDMICH TODO: eliminate all usage of new_custom_columns
-      static std::vector<custom_col_t> new_custom_columns; //and instead let the arranger update without restarting muse!
+      static std::vector<custom_col_t> custom_columns;
 
       Arranger(ArrangerView* parent, const char* name = 0);
 
@@ -230,17 +185,22 @@ class Arranger : public QWidget {
 
       MusECore::Track* curTrack() const { return selected; }
       void cmd(int);
-      bool isSingleSelection() { return canvas->isSingleSelection(); }
-      int selectionSize() { return canvas->selectionSize(); }
+      bool isSingleSelection() const { return canvas->isSingleSelection(); }
+      int selectionSize() const { return canvas->selectionSize(); }
+      bool itemsAreSelected() const { return canvas->itemsAreSelected(); }
       void setGlobalTempo(int);
       void clear();
       void songIsClearing() { canvas->songIsClearing(); }
+      void setDefaultSplitterSizes();
+      void updateHeaderCustomColumns();
       
       unsigned cursorValue() { return cursVal; }
       
       ArrangerView* parentWin() const { return _parentWin; }
 
       bool setRasterVal(int);
+
+      TList *getTrackList() { return list; }
       };
 
 } // namespace MusEGui
