@@ -28,6 +28,10 @@
 
 #include "value.h"
 #include "mtc.h"
+#include "metronome_class.h"
+#include "wave.h"
+#include "audio_convert/audio_converter_plugin.h"
+#include "audio_convert/audio_converter_settings_group.h"
 
 #include <unistd.h>
 
@@ -35,6 +39,8 @@ class QString;
 class QAction;
 class QActionGroup;
 class QTimer;
+class QToolButton;
+
 
 namespace MusEGui {
 class MusE;
@@ -42,23 +48,41 @@ class MusE;
 
 namespace MusEGlobal {
 
-enum ClickSamples {
-    origSamples,
-    newSamples
-};
-
 extern const float denormalBias;
 
 extern int sampleRate;
 extern unsigned segmentSize;
 extern unsigned fifoLength; // inversely proportional to segmentSize
 extern int segmentCount;
+extern int projectSampleRate;
+extern const int numAudioSampleRates;
+extern const int selectableAudioSampleRates[];
+
+extern MusECore::SndFileList sndFiles;
+
+extern MusECore::AudioConverterPluginList audioConverterPluginList;
+// This global variable is a pointer so that we can replace it quickly with a new one in RT operations.
+extern MusECore::AudioConverterSettingsGroup* defaultAudioConverterSettings;
 
 extern bool overrideAudioOutput;
 extern bool overrideAudioInput;
 
+extern const QString selectableAudioBackendDevices[];
+extern const int numRtAudioDevices;
+enum SelectableAudioBackendDevices {
+  JackAudio = 0,
+  DummyAudio = 1,
+  RtAudioPulse = 2,
+  RtAudioAlsa = 3,
+  RtAudioOss = 4,
+  RtAudioChoice = 5
+  //RtAudioJack, -- it's just stupid to keep this option
+};
+
 extern QTimer* heartBeatTimer;
 
+extern bool blinkTimerPhase;
+
 extern bool hIsB;
 
 extern const signed char sharpTab[14][7];
@@ -73,6 +97,7 @@ extern QString configName;
 extern QString configPath;
 extern QString museInstruments;
 extern QString museUserInstruments;
+extern QString cachePath;
 
 extern QString lastWavePath;
 extern QString lastMidiPath;
@@ -85,13 +110,21 @@ extern bool debugMsg;
 extern bool heavyDebugMsg;
 extern bool debugSync;
 extern bool loadPlugins;
+extern bool loadMESS;
 extern bool loadVST;
 extern bool loadNativeVST;
 extern bool loadDSSI;
 extern bool usePythonBridge;
+extern QString pythonBridgePyroNSHostname;
+extern QString pythonBridgePyroNSPort;
+extern QString pythonBridgePyroDaemonHostname;
+extern QString pythonBridgePyroDaemonPort;
+extern float pythonBridgePyroCommTimeout;
 extern bool useLASH;
+extern bool loadLV2;
 extern bool useAlsaWithJack;
 extern bool noAutoStartJack;
+extern bool populateMidiPortsOnStart;
 
 extern bool realTimeScheduling;
 extern int realTimePriority;
@@ -110,6 +143,8 @@ extern const char* preset_file_save_pattern[];
 extern const char* drum_map_file_pattern[];
 extern const char* drum_map_file_save_pattern[];
 extern const char* audio_file_pattern[];
+extern const char* colors_config_file_pattern[];
+extern const char* stylesheet_file_pattern[];
 
 extern Qt::KeyboardModifiers globalKeyState;
 
@@ -151,35 +186,15 @@ extern QAction* punchoutAction;
 extern QAction* recordAction;
 extern QAction* panicAction;
 extern QAction* metronomeAction;
+extern QAction* cpuLoadAction;
 
 extern MusEGui::MusE* muse;
 
-extern int preMeasures;
-extern unsigned char measureClickNote;
-extern unsigned char measureClickVelo;
-extern unsigned char beatClickNote;
-extern unsigned char beatClickVelo;
-extern unsigned char accentClick1;
-extern unsigned char accentClick1Velo;
-extern unsigned char accentClick2;
-extern unsigned char accentClick2Velo;
-
-extern unsigned char clickChan;
-extern unsigned char clickPort;
-extern bool precountEnableFlag;
-extern bool precountFromMastertrackFlag;
-extern int precountSigZ;
-extern int precountSigN;
-extern bool precountPrerecord;
-extern bool precountPreroll;
-extern bool midiClickFlag;
-extern bool audioClickFlag;
-extern float audioClickVolume;
-extern float measClickVolume;
-extern float beatClickVolume;
-extern float accent1ClickVolume;
-extern float accent2ClickVolume;
-extern ClickSamples clickSamples;
+extern MusECore::MetroAccentsPresetsMap metroAccentPresets;
+extern MusECore::MetronomeSettings metroGlobalSettings;
+extern MusECore::MetronomeSettings metroSongSettings;
+// Whether to use the global or song metronome settings.
+extern bool metroUseSongSettings;
 
 extern bool rcEnable;
 extern unsigned char rcStopNote;
@@ -195,16 +210,25 @@ extern int midiLearnCtrl;
 extern bool midiSeqRunning;
 extern bool automation;
 extern int trackHeight;
-// p3.3.55
+
+extern const QString inputRoutingToolTipBase;
+extern const QString outputRoutingToolTipBase;
+extern const QString noInputRoutingToolTipWarn;
+extern const QString noOutputRoutingToolTipWarn;
+
+
 #define JACK_MIDI_OUT_PORT_SUFFIX "_out"
 #define JACK_MIDI_IN_PORT_SUFFIX  "_in"
 
-extern uid_t euid, ruid;
+// REMOVE Tim. setuid. Removed.
+// extern uid_t euid, ruid;
 extern void doSetuid();
 extern void undoSetuid();
 extern bool checkAudioDevice();
 extern bool getUniqueTmpfileName(QString subDir, QString ext, QString& newFilename);
 
+extern unsigned convertFrame4ProjectSampleRate(unsigned frame);
+
 } // namespace MusEGlobal
 
 #endif
