@@ -86,7 +86,20 @@ void MusE::importMidi(const QString &file)
 
       switch (n) {
             case 0:
+                  // FIXME TODO REMOVE Tim. clip. Added. Comment.
+                  // Hm, importMidi() doesn't even stop the audio or idle or anything! Unsafe?
+                  // Do like in loadProjectFile() ... Hm, don't think we can stop the audio
+                  //  because it needs to process commands below.
+                  //
+                  // Try this as a minimum...
+                  stopHeartBeat();
+                  MusEGlobal::audio->msgIdle(true);
+
                   importMidi(fn, true);
+
+                  MusEGlobal::audio->msgIdle(false);
+                  setHeartBeat();
+
                   MusEGlobal::song->update();
                   break;
             case 1:
@@ -112,7 +125,7 @@ bool MusE::importMidi(const QString name, bool merge)
       bool rv = mf.read();
       popenFlag ? pclose(fp) : fclose(fp);
       if (rv) {
-            QString s(tr("reading midifile\n  "));
+            QString s(tr("Reading midifile\n  "));
             s += name;
             s += tr("\nfailed: ");
             s += mf.error();
@@ -145,7 +158,7 @@ bool MusE::importMidi(const QString name, bool merge)
       bool dev_changed = false;
       for(MusECore::iMidiFilePort imp = usedPortMap->begin(); imp != usedPortMap->end(); ++imp) 
       {
-        MType midi_type = imp->second._midiType;
+        MusECore::MType midi_type = imp->second._midiType;
         QString instr_name = MusEGlobal::config.importInstrNameMetas ? imp->second._instrName : QString();
         MusECore::MidiInstrument* typed_instr = 0;
         MusECore::MidiInstrument* named_instr = 0;
@@ -153,11 +166,11 @@ bool MusE::importMidi(const QString name, bool merge)
         for(MusECore::iMidiInstrument i = MusECore::midiInstruments.begin(); i != MusECore::midiInstruments.end(); ++i) 
         {
           MusECore::MidiInstrument* mi = *i;
-          if(midi_type != MT_UNKNOWN && midi_type == mi->midiType())
+          if(midi_type != MusECore::MT_UNKNOWN && midi_type == mi->midiType())
             typed_instr = mi;
           if(!instr_name.isEmpty() && instr_name == mi->iname())
             named_instr = mi;
-          if((typed_instr && named_instr) || ((typed_instr && instr_name.isEmpty()) || (named_instr && midi_type == MT_UNKNOWN)))
+          if((typed_instr && named_instr) || ((typed_instr && instr_name.isEmpty()) || (named_instr && midi_type == MusECore::MT_UNKNOWN)))
             break;  // Done searching
         }
 
@@ -174,8 +187,9 @@ bool MusE::importMidi(const QString name, bool merge)
             if(md)
             {
               // TEST: Hopefully shouldn't require any routing saves/restorations as in midi port config set device name...
-              MusEGlobal::midiSeq->msgSetMidiDevice(mp, md);
+              MusEGlobal::audio->msgSetMidiDevice(mp, md);
               // TEST: Hopefully can get away with this ouside the loop below...
+              // Save settings. Use simple version - do NOT set style or stylesheet, this has nothing to do with that.
               //MusEGlobal::muse->changeConfig(true);     // save configuration file
               //MusEGlobal::audio->msgUpdateSoloStates();
               //MusEGlobal::song->update();
@@ -222,7 +236,7 @@ bool MusE::importMidi(const QString name, bool merge)
         // If the instrument is one of the three standard GM, GS, or XG, mark the usedPort as "ch 10 is drums".
         // Otherwise it's anybody's guess what channel(s) drums are on.
         // Code is a bit HACKISH just to accomplish passing this bool value to the next stage, where tracks are created. 
-        if(instr->midiType() != MT_UNKNOWN)
+        if(instr->midiType() != MusECore::MT_UNKNOWN)
           imp->second._isStandardDrums = true;
           
         // Set the device's instrument - ONLY for non-synths because they provide their own. 
@@ -230,7 +244,7 @@ bool MusE::importMidi(const QString name, bool merge)
         {
           // this overwrites any instrument set for this port:
           if(mp->instrument() != instr)
-            mp->setInstrument(instr);
+            mp->changeInstrument(instr);
         }
       }
       
@@ -238,7 +252,8 @@ bool MusE::importMidi(const QString name, bool merge)
       {
         // TEST: Hopefully can get away with this here instead of inside the loop above...
         // TEST: Are these really necessary as in midi port config set device name?
-        MusEGlobal::muse->changeConfig(true);     // save configuration file 
+        // Save settings. Use simple version - do NOT set style or stylesheet, this has nothing to do with that.
+        MusEGlobal::muse->changeConfig(true);     // save configuration file
         MusEGlobal::audio->msgUpdateSoloStates(); // 
         MusEGlobal::song->update();
       }
@@ -249,8 +264,8 @@ bool MusE::importMidi(const QString name, bool merge)
       //    - calculate tick value for internal resolution
       //
       for (MusECore::iMidiFileTrack t = etl->begin(); t != etl->end(); ++t) {
-            MusECore::MPEventList* el  = &((*t)->events);
-            if (el->empty())
+            MusECore::MPEventList& el  = ((*t)->events);
+            if (el.empty())
                   continue;
             //
             // if we split the track, SYSEX and META events go into
@@ -264,7 +279,7 @@ bool MusE::importMidi(const QString name, bool merge)
             
             MusECore::iMPEvent ev;
             set< pair<int,int> > already_processed;
-            for (ev = el->begin(); ev != el->end(); ++ev)
+            for (ev = el.begin(); ev != el.end(); ++ev)
             {
               if (ev->type() != MusECore::ME_SYSEX && ev->type() != MusECore::ME_META)
               {
@@ -278,19 +293,18 @@ bool MusE::importMidi(const QString name, bool merge)
                         MusECore::MidiTrack* track = new MusECore::MidiTrack();
                         if ((*t)->_isDrumTrack)
                         {
-                           if (MusEGlobal::config.importMidiNewStyleDrum)
+// Obsolete. There is only 'New' drum tracks now.
+//                            if (MusEGlobal::config.importMidiNewStyleDrum)
                               track->setType(MusECore::Track::NEW_DRUM);
-                           else
-                              track->setType(MusECore::Track::DRUM);
+//                            else
+//                               track->setType(MusECore::Track::DRUM);
                         }
                               
                         track->setOutChannel(channel);
                         track->setOutPort(port);
 
-                        MusECore::MidiPort* mport = &MusEGlobal::midiPorts[port];
-                        //MusECore::MidiInstrument* instr = mport->instrument();
-                        MusECore::EventList* mel = track->events();
-                        buildMidiEventList(mel, el, track, division, first, false); // Don't do loops.
+//                         MusECore::MidiPort* mport = &MusEGlobal::midiPorts[port];
+                        buildMidiEventList(&track->events, el, track, division, first, false); // Don't do loops.
                         first = false;
 
                         // Comment Added by T356.
@@ -299,29 +313,29 @@ bool MusE::importMidi(const QString name, bool merge)
                         //if (channel == 9 && instr->midiType() != MT_UNKNOWN) {
                         MusECore::ciMidiFilePort imp = usedPortMap->find(port);
                         if(imp != usedPortMap->end() && imp->second._isStandardDrums && channel == 9) { // A bit HACKISH, see above
-                           if (MusEGlobal::config.importMidiNewStyleDrum)
+// Obsolete. There is only 'New' drum tracks now.
+//                            if (MusEGlobal::config.importMidiNewStyleDrum)
                               track->setType(MusECore::Track::NEW_DRUM);
-                           else
-                           {
-                              track->setType(MusECore::Track::DRUM);
-                              // remap drum pitch with drumOutmap (was: Inmap. flo93 thought this was wrong)
-                              MusECore::EventList* tevents = track->events();
-                              for (MusECore::iEvent i = tevents->begin(); i != tevents->end(); ++i) {
-                                    MusECore::Event ev  = i->second;
-                                    if (ev.isNote()) {
-                                          int pitch = MusEGlobal::drumOutmap[ev.pitch()];
-                                          ev.setPitch(pitch);
-                                          }
-                                    else
-                                    if(ev.type() == MusECore::Controller)
-                                    {
-                                      int ctl = ev.dataA();
-                                      MusECore::MidiController *mc = mport->drumController(ctl);
-                                      if(mc)
-                                        ev.setA((ctl & ~0xff) | MusEGlobal::drumOutmap[ctl & 0x7f]);
-                                    }
-                              }
-                           }
+//                            else
+//                            {
+//                               track->setType(MusECore::Track::DRUM);
+//                               // remap drum pitch with drumOutmap (was: Inmap. flo93 thought this was wrong)
+//                               for (MusECore::iEvent i = track->events.begin(); i != track->events.end(); ++i) {
+//                                     MusECore::Event ev  = i->second;
+//                                     if (ev.isNote()) {
+//                                           int pitch = MusEGlobal::drumOutmap[ev.pitch()];
+//                                           ev.setPitch(pitch);
+//                                           }
+//                                     else
+//                                     if(ev.type() == MusECore::Controller)
+//                                     {
+//                                       int ctl = ev.dataA();
+//                                       MusECore::MidiController *mc = mport->drumController(ctl);
+//                                       if(mc)
+//                                         ev.setA((ctl & ~0xff) | MusEGlobal::drumOutmap[ctl & 0x7f]);
+//                                     }
+//                               }
+//                            }
                         }
                               
                         processTrack(track);
@@ -339,8 +353,7 @@ bool MusE::importMidi(const QString name, bool merge)
                   MusECore::MidiTrack* track = new MusECore::MidiTrack();
                   track->setOutChannel(0);
                   track->setOutPort(0);
-                  MusECore::EventList* mel = track->events();
-                  buildMidiEventList(mel, el, track, division, true, false); // Do SysexMeta. Don't do loops.
+                  buildMidiEventList(&track->events, el, track, division, true, false); // Do SysexMeta. Don't do loops.
                   processTrack(track);
                   MusEGlobal::song->insertTrack0(track, -1);
                   }
@@ -355,7 +368,7 @@ bool MusE::importMidi(const QString name, bool merge)
             MusEGlobal::song->initLen();
 
             int z, n;
-            AL::sigmap.timesig(0, z, n);
+            MusEGlobal::sigmap.timesig(0, z, n);
 
             int tempo = MusEGlobal::tempomap.tempo(0);
             transport->setTimesig(z, n);
@@ -363,7 +376,6 @@ bool MusE::importMidi(const QString name, bool merge)
 
             bool masterF = !MusEGlobal::tempomap.empty();
             MusEGlobal::song->setMasterFlag(masterF);
-            transport->setMasterFlag(masterF);
 
             MusEGlobal::song->updatePos();
 
@@ -383,8 +395,8 @@ bool MusE::importMidi(const QString name, bool merge)
 
 void MusE::processTrack(MusECore::MidiTrack* track)
       {
-      MusECore::EventList* tevents = track->events();
-      if (tevents->empty())
+      MusECore::EventList& tevents = track->events;
+      if (tevents.empty())
             return;
 
       //---------------------------------------------------
@@ -398,8 +410,8 @@ void MusE::processTrack(MusECore::MidiTrack* track)
       MusECore::PartList* pl = track->parts();
 
       int lastTick = 0;
-      for (MusECore::iEvent i = tevents->begin(); i != tevents->end(); ++i) {
-            MusECore::Event event = i->second;
+      for (MusECore::ciEvent i = tevents.begin(); i != tevents.end(); ++i) {
+            const MusECore::Event& event = i->second;
             int epos = event.tick() + event.lenTick();
             if (epos > lastTick)
                   lastTick = epos;
@@ -414,7 +426,7 @@ void MusE::processTrack(MusECore::MidiTrack* track)
         
         int bar2, beat;
         unsigned tick;
-        AL::sigmap.tickValues(len, &bar2, &beat, &tick);
+        MusEGlobal::sigmap.tickValues(len, &bar2, &beat, &tick);
         
         int lastOff = 0;
         int st = -1;      // start tick current part
@@ -422,12 +434,12 @@ void MusE::processTrack(MusECore::MidiTrack* track)
         int x2 = 0;       // end tick current measure
   
         for (int bar = 0; bar < bar2; ++bar, x1 = x2) {
-              x2 = AL::sigmap.bar2tick(bar+1, 0, 0);
+              x2 = MusEGlobal::sigmap.bar2tick(bar+1, 0, 0);
               if (lastOff > x2) {        
                     continue;
                     }
-              MusECore::iEvent i1 = tevents->lower_bound(x1);
-              MusECore::iEvent i2 = tevents->lower_bound(x2);
+              MusECore::iEvent i1 = tevents.lower_bound(x1);
+              MusECore::iEvent i2 = tevents.lower_bound(x2);
   
               if (i1 == i2) {   // empty?
                     if (st != -1) {
@@ -444,8 +456,8 @@ void MusE::processTrack(MusECore::MidiTrack* track)
                           st = x1;    // begin new  part
                     //HACK:
                     //lastOff:
-                    for (MusECore::iEvent i = i1; i != i2; ++i) {
-                          MusECore::Event event = i->second;
+                    for (MusECore::ciEvent i = i1; i != i2; ++i) {
+                          const MusECore::Event& event = i->second;
                           if (event.type() == MusECore::Note) {
                                 int off = event.tick() + event.lenTick();
                                 if (off > lastOff)
@@ -480,28 +492,27 @@ void MusE::processTrack(MusECore::MidiTrack* track)
             MusECore::MidiPart* part = (MusECore::MidiPart*)(p->second);
             int stick = part->tick();
             int etick = part->tick() + part->lenTick();
-            MusECore::iEvent r1 = tevents->lower_bound(stick);
-            MusECore::iEvent r2 = tevents->lower_bound(etick);
+            MusECore::iEvent r1 = tevents.lower_bound(stick);
+            MusECore::iEvent r2 = tevents.lower_bound(etick);
             int startTick = part->tick();
 
-            MusECore::EventList* el = part->events();
             for (MusECore::iEvent i = r1; i != r2; ++i) {
-                  MusECore::Event ev = i->second;
+                  MusECore::Event& ev = i->second;
                   int ntick = ev.tick() - startTick;
                   ev.setTick(ntick);
-                  el->add(ev);
+                  part->addEvent(ev);
                   }
-            tevents->erase(r1, r2);
+            tevents.erase(r1, r2);
             }
 
-      if (tevents->size())
-            printf("-----------events left: %zd\n", tevents->size());
-      for (MusECore::iEvent i = tevents->begin(); i != tevents->end(); ++i) {
+      if (tevents.size())
+            printf("-----------events left: %zd\n", tevents.size());
+      for (MusECore::ciEvent i = tevents.begin(); i != tevents.end(); ++i) {
             printf("%d===\n", i->first);
             i->second.dump();
             }
       // all events should be processed:
-      if (!tevents->empty())
+      if (!tevents.empty())
         printf("THIS SHOULD NEVER HAPPEN: not all events processed at the end of MusE::processTrack()!\n");
       }
 
@@ -620,7 +631,7 @@ void MusE::importPartToTrack(QString& filename, unsigned tick, MusECore::Track*
                   if (tag == "part") {
                         // Read the part.
                         MusECore::Part* p = 0;
-                        p = readXmlPart(xml, track);
+                        p = MusECore::Part::readFromXml(xml, track);
                         // If it could not be created...
                         if(!p)
                         {
@@ -638,7 +649,9 @@ void MusE::importPartToTrack(QString& filename, unsigned tick, MusECore::Track*
                           posOffset = tick - p->tick();
                         }
                         p->setTick(p->tick() + posOffset);
-                        MusEGlobal::audio->msgAddPart(p,false);
+                        // Operation is undoable but do not start/end undo.
+                        MusEGlobal::song->applyOperation(MusECore::UndoOp(MusECore::UndoOp::AddPart, p),
+                                                         MusECore::Song::OperationUndoable);
                         }
                   else
                         xml.unknown("MusE::importPartToTrack");
