@@ -26,7 +26,9 @@
 #include <limits.h>
 #include <sys/stat.h>
 #include <sys/types.h>
+#ifndef _WIN32
 #include <sys/mman.h>
+#endif
 
 #include <QKeyEvent>
 #include <QDropEvent>
@@ -34,6 +36,7 @@
 #include <QMimeData>
 #include <QByteArray>
 #include <QDrag>
+#include <QSet>
 
 #include "xml.h"
 #include "midieditor.h"
@@ -63,10 +66,6 @@ EventCanvas::EventCanvas(MidiEditor* pr, QWidget* parent, int sx,
       _playEvents = true;
       _setCurPartIfOnlyOneEventIsSelected = true;
       curVelo     = 70;
-      playedPitch = -1;
-      playedPitchChannel = -1;
-      playedPitchPort = -1;
-      playedVelocity = 0;
 
       setBg(MusEGlobal::config.midiCanvasBg);
       setAcceptDrops(true);
@@ -91,8 +90,8 @@ QString EventCanvas::getCaption() const
       {
       int bar1, bar2, xx;
       unsigned x;
-      AL::sigmap.tickValues(curPart->tick(), &bar1, &xx, &x);
-      AL::sigmap.tickValues(curPart->tick() + curPart->lenTick(), &bar2, &xx, &x);
+      MusEGlobal::sigmap.tickValues(curPart->tick(), &bar1, &xx, &x);
+      MusEGlobal::sigmap.tickValues(curPart->tick() + curPart->lenTick(), &bar2, &xx, &x);
 
       return QString("MusE: Part <") + curPart->name()
          + QString("> %1-%2").arg(bar1+1).arg(bar2+1);
@@ -137,81 +136,155 @@ QPoint EventCanvas::raster(const QPoint& p) const
 //---------------------------------------------------------
 
 void EventCanvas::mouseMove(QMouseEvent* event)
-      {
-      emit pitchChanged(y2pitch(event->pos().y()));
-      int x = event->pos().x();
-      emit timeChanged(editor->rasterVal(x));
-      }
+{
+    emit pitchChanged(y2pitch(event->pos().y()));
+    int x = event->pos().x();
+    if(x < 0)
+        x = 0;
+    emit timeChanged(editor->rasterVal(x));
+}
 
 //---------------------------------------------------------
-//   updateSelection
+//   updateItems
 //---------------------------------------------------------
 
-void EventCanvas::updateSelection()
+void EventCanvas::updateItems()
+{
+  bool curItemNeedsRestore=false;
+  MusECore::Event storedEvent;
+  int partSn = 0xDEADBEEF; // to prevent compiler warning; partSn is unused anyway if curItemNeedsRestore==false.
+  if (curItem)
+  {
+    curItemNeedsRestore=true;
+    storedEvent=curItem->event();
+    partSn=curItem->part()->sn();
+  }
+  curItem=NULL;
+  
+  items.clearDelete();
+  start_tick  = INT_MAX;
+  end_tick    = 0;
+  curPart = 0;
+  for (MusECore::iPart p = editor->parts()->begin(); p != editor->parts()->end(); ++p) {
+        MusECore::MidiPart* part = (MusECore::MidiPart*)(p->second);
+        if (part->sn() == curPartId)
+              curPart = part;
+        unsigned stick = part->tick();
+        unsigned len = part->lenTick();
+        unsigned etick = stick + len;
+        if (stick < start_tick)
+              start_tick = stick;
+        if (etick > end_tick)
+              end_tick = etick;
+
+        for (MusECore::ciEvent i = part->events().begin(); i != part->events().end(); ++i) {
+              MusECore::Event e = i->second;
+              // Do not add events which are past the end of the part.
+              if(e.tick() > len)      
+                break;
+              
+              if (e.isNote()) {
+                    CItem* temp = addItem(part, e);
+                    
+                    if(temp)
+                      temp->setSelected(e.selected());
+                    
+                    if (temp && curItemNeedsRestore && e==storedEvent && part->sn()==partSn)
+                    {
+                        if (curItem!=NULL)
+                          fprintf(stderr, "THIS SHOULD NEVER HAPPEN: curItemNeedsRestore=true, event fits, but there was already a fitting event!?\n");
+                        
+                        curItem=temp;
+                        }
+                    }
+              }
+        }
+}
+
+//---------------------------------------------------------
+//   itemSelectionsChanged
+//---------------------------------------------------------
+
+bool EventCanvas::itemSelectionsChanged(MusECore::Undo* operations, bool deselectAll)
+{
+      MusECore::Undo ops;
+      MusECore::Undo* opsp = operations ? operations : &ops;
+      
+      bool item_selected;
+      bool obj_selected;
+      bool changed=false;
+      
+      // If we are deselecting all, globally deselect all events,
+      //  and don't bother individually deselecting objects, below.
+      if(deselectAll)
       {
-      MusEGlobal::song->update(SC_SELECTION);
+        opsp->push_back(MusECore::UndoOp(MusECore::UndoOp::GlobalSelectAllEvents, false, 0, 0));
+        changed = true;
       }
+      
+      
+      for (iCItem i = items.begin(); i != items.end(); ++i) {
+            CItem* item = i->second;
+            item_selected = item->isSelected();
+            obj_selected = item->objectIsSelected();
+            // Don't bother deselecting objects if we have already deselected all, above.
+            if((item_selected || !deselectAll) &&
+                ((item_selected != obj_selected) ||
+                // Need to force this because after the 'deselect all events' command executes,
+                //  if the item is selected another select needs to be executed even though it
+                //  appears nothing changed here.
+                (item_selected && deselectAll)))
+              
+            {
+              opsp->push_back(MusECore::UndoOp(MusECore::UndoOp::SelectEvent,
+                  item->event(), item->part(), item_selected, obj_selected));
+
+              changed=true;
+            }
+      }
+
+      if (!operations && changed)
+      {
+            // Set the 'sender' to this so that we can ignore self-generated songChanged signals.
+            // Here we have a choice of whether to allow undoing of selections.
+            if(MusEGlobal::config.selectionsUndoable)
+              MusEGlobal::song->applyOperationGroup(ops, MusECore::Song::OperationUndoMode, this);
+            else
+              MusEGlobal::song->applyOperationGroup(ops, MusECore::Song::OperationExecuteUpdate, this);
+            
+// For testing...
+//               fprintf(stderr, "EventCanvas::updateSelection: Applied SelectPart operations, redrawing\n");
+      }
+
+      return changed;
+}
+
+
+bool EventCanvas::stuckNoteExists(int port, int channel, int pitch) const
+{
+  const int sz = _stuckNotes.size();
+  for(int i = 0; i < sz; ++i)
+  {
+    MusECore::MidiPlayEvent s_ev(_stuckNotes.at(i));
+    if(s_ev.type() == MusECore::ME_NOTEON &&
+       port == s_ev.port() &&
+       channel == s_ev.channel() &&
+       pitch == s_ev.dataA())
+      return true;
+  }
+  return false;
+}
 
 //---------------------------------------------------------
 //   songChanged(type)
 //---------------------------------------------------------
 
-void EventCanvas::songChanged(MusECore::SongChangedFlags_t flags)
+void EventCanvas::songChanged(MusECore::SongChangedStruct_t flags)
       {
-      // Is it simply a midi controller value adjustment? Forget it.
-      if(flags == SC_MIDI_CONTROLLER)
-        return;
-    
-      if (flags & ~SC_SELECTION) {
+      if (flags & ~(SC_SELECTION | SC_PART_SELECTION | SC_TRACK_SELECTION)) {
             // TODO FIXME: don't we actually only want SC_PART_*, and maybe SC_TRACK_DELETED?
             //             (same in waveview.cpp)
-            bool curItemNeedsRestore=false;
-            MusECore::Event storedEvent;
-            int partSn = 0xDEADBEEF; // to prevent compiler warning; partSn is unused anyway if curItemNeedsRestore==false.
-            if (curItem)
-            {
-              curItemNeedsRestore=true;
-              storedEvent=curItem->event();
-              partSn=curItem->part()->sn();
-            }
-            curItem=NULL;
-            
-            items.clearDelete();
-            start_tick  = INT_MAX;
-            end_tick    = 0;
-            curPart = 0;
-            for (MusECore::iPart p = editor->parts()->begin(); p != editor->parts()->end(); ++p) {
-                  MusECore::MidiPart* part = (MusECore::MidiPart*)(p->second);
-                  if (part->sn() == curPartId)
-                        curPart = part;
-                  unsigned stick = part->tick();
-                  unsigned len = part->lenTick();
-                  unsigned etick = stick + len;
-                  if (stick < start_tick)
-                        start_tick = stick;
-                  if (etick > end_tick)
-                        end_tick = etick;
-
-                  MusECore::EventList* el = part->events();
-                  for (MusECore::iEvent i = el->begin(); i != el->end(); ++i) {
-                        MusECore::Event e = i->second;
-                        // Do not add events which are past the end of the part.
-                        if(e.tick() > len)      
-                          break;
-                        
-                        if (e.isNote()) {
-                              CItem* temp = addItem(part, e);
-                              
-                              if (temp && curItemNeedsRestore && e==storedEvent && part->sn()==partSn)
-                              {
-                                  if (curItem!=NULL)
-                                    printf("THIS SHOULD NEVER HAPPEN: curItemNeedsRestore=true, event fits, but there was already a fitting event!?\n");
-                                  
-                                  curItem=temp;
-                                  }
-                              }
-                        }
-                  }
+            updateItems();
             }
 
       MusECore::Event event;
@@ -222,14 +295,12 @@ void EventCanvas::songChanged(MusECore::SongChangedFlags_t flags)
       int n  = 0;       // count selections
       for (iCItem k = items.begin(); k != items.end(); ++k) {
             MusECore::Event ev = k->second->event();
-            bool selected = ev.selected();
-            if (selected) {
-                  k->second->setSelected(true);
+            
+            if (ev.selected()) {
                   ++n;
                   if (!nevent) {
                         nevent   =  k->second;
-                        MusECore::Event mi = nevent->event();
-                        curVelo  = mi.velo();
+                        curVelo = ev.velo();
                         }
                   }
             }
@@ -247,13 +318,22 @@ void EventCanvas::songChanged(MusECore::SongChangedFlags_t flags)
                   curPartChanged();
                   }
       }
+
+      if(flags & (SC_SELECTION))
+      {
+        // Prevent race condition: Ignore if the change was ultimately sent by the canvas itself.
+        if(flags._sender != this)
+          updateItemSelections();
+      }
       
-      bool f1 = flags & (SC_EVENT_INSERTED | SC_EVENT_MODIFIED | SC_EVENT_REMOVED | 
+      bool f1 = static_cast<bool>(flags & (SC_EVENT_INSERTED | SC_EVENT_MODIFIED | SC_EVENT_REMOVED | 
                          SC_PART_INSERTED | SC_PART_MODIFIED | SC_PART_REMOVED |
                          SC_TRACK_INSERTED | SC_TRACK_REMOVED | SC_TRACK_MODIFIED |
-                         SC_SIG | SC_TEMPO | SC_KEY | SC_MASTER | SC_CONFIG | SC_DRUMMAP); 
-      bool f2 = flags & SC_SELECTION;
-      if(f1 || f2)   // Try to avoid all unnecessary emissions.
+                         SC_SIG | SC_TEMPO | SC_KEY | SC_MASTER | SC_CONFIG | SC_DRUMMAP));
+      bool f2 = static_cast<bool>(flags & SC_SELECTION);
+      
+      // Try to avoid all unnecessary emissions.
+      if(f1 || f2)
         emit selectionChanged(x, event, part, !f1);
       
       if (curPart == 0)
@@ -265,37 +345,39 @@ void EventCanvas::songChanged(MusECore::SongChangedFlags_t flags)
 //   selectAtTick
 //---------------------------------------------------------
 void EventCanvas::selectAtTick(unsigned int tick)
-      {
-      //Select note nearest tick, if none selected and there are any
-      if (!items.empty() && selectionSize() == 0) {
-            iCItem i = items.begin();
-	    CItem* nearest = i->second;
+{
+   //Select note nearest tick, if none selected and there are any
+   if (!items.empty() && selectionSize() == 0) {
+      iCItem i = items.begin();
+      CItem* nearest = i->second;
 
-            while (i != items.end()) {
-                CItem* cur=i->second;                
-                unsigned int curtk=abs(cur->x() + cur->part()->tick() - tick);
-                unsigned int neartk=abs(nearest->x() + nearest->part()->tick() - tick);
+      while (i != items.end()) {
+         CItem* cur=i->second;
+         unsigned int curtk=abs(cur->x() + (int)cur->part()->tick() - (int)tick);
+         unsigned int neartk=abs(nearest->x() + (int)nearest->part()->tick() - (int)tick);
 
-                if (curtk < neartk) {
-                    nearest=cur;
-                    }
+         if (curtk < neartk) {
+            nearest=cur;
+         }
 
-                i++;
-                }
-
-            if (!nearest->isSelected()) {
-                  selectItem(nearest, true);
-                  songChanged(SC_SELECTION);
-                  }
-            }
+         i++;
       }
 
+      if (!nearest->isSelected()) {
+         selectItem(nearest, true);
+         songChanged(SC_SELECTION);
+      }
+   }
+}
+
 //---------------------------------------------------------
 //   track
 //---------------------------------------------------------
 
 MusECore::MidiTrack* EventCanvas::track() const
       {
+      if(!curPart)
+        return 0;
       return ((MusECore::MidiPart*)curPart)->track();
       }
 
@@ -338,8 +420,8 @@ void EventCanvas::keyPress(QKeyEvent* event)
             if (found) {
                   MusECore::Pos p1(tick_min, true);
                   MusECore::Pos p2(tick_max, true);
-                  MusEGlobal::song->setPos(1, p1);
-                  MusEGlobal::song->setPos(2, p2);
+                  MusEGlobal::song->setPos(MusECore::Song::LPOS, p1);
+                  MusEGlobal::song->setPos(MusECore::Song::RPOS, p2);
                   }
             }
       // Select items by key (PianoRoll & DrumEditor)
@@ -363,7 +445,8 @@ void EventCanvas::keyPress(QKeyEvent* event)
                       deselectAll();
                 CItem* sel = i->second;
                 sel->setSelected(true);
-                updateSelection();
+                redraw();
+                
                 if (sel->x() + sel->width() > mapxDev(width())) 
                 {  
                   int mx = rmapx(sel->x());  
@@ -393,7 +476,7 @@ void EventCanvas::keyPress(QKeyEvent* event)
                       deselectAll();
                 CItem* sel = i->second;
                 sel->setSelected(true);
-                updateSelection();
+                redraw();
                 if (sel->x() <= mapxDev(0)) 
                   emit horizontalScroll(rmapx(sel->x() - xorg) - 10);  // Leave a bit of room.
               }
@@ -427,6 +510,36 @@ void EventCanvas::keyPress(QKeyEvent* event)
       }
 
 
+//---------------------------------------------------------
+//   keyRelease
+//---------------------------------------------------------
+
+void EventCanvas::keyRelease(QKeyEvent* event)
+{
+      const int key = event->key();
+      
+      // We do not want auto-repeat events.
+      // It does press and release repeatedly. Wait till the last release comes.
+      if(!event->isAutoRepeat())
+      {
+// For testing...
+//         fprintf(stderr, "EventCanvas::keyRelease not isAutoRepeat\n");
+      
+        //event->accept();
+      
+        // Select part to the right
+        if(key == shortcuts[SHRT_SEL_RIGHT].key || key == shortcuts[SHRT_SEL_RIGHT_ADD].key ||
+        // Select part to the left
+          key == shortcuts[SHRT_SEL_LEFT].key || key == shortcuts[SHRT_SEL_LEFT_ADD].key)
+        {
+            itemSelectionsChanged();
+        }
+        return;
+      }
+      
+  Canvas::keyRelease(event);
+}
+
 //---------------------------------------------------------
 //   dropEvent
 //---------------------------------------------------------
@@ -435,7 +548,7 @@ void EventCanvas::viewDropEvent(QDropEvent* event)
       {
       QString text;
       if (event->source() == this) {
-            printf("local DROP\n");      
+            fprintf(stderr, "local DROP\n");      
             //event->acceptProposedAction();     
             //event->ignore();                     // TODO CHECK Tim.
             return;
@@ -443,14 +556,17 @@ void EventCanvas::viewDropEvent(QDropEvent* event)
       if (event->mimeData()->hasFormat("text/x-muse-groupedeventlists")) {
             text = QString(event->mimeData()->data("text/x-muse-groupedeventlists"));
       
-            int x = editor->rasterVal(event->pos().x());
+            int x = event->pos().x();
+            if(x < 0)
+              x = 0;
+            x = editor->rasterVal(x);
             if (x < 0)
                   x = 0;
             paste_at(text,x,3072,false,false,curPart);
             //event->accept();  // TODO
             }
       else {
-            printf("cannot decode drop\n");
+            fprintf(stderr, "cannot decode drop\n");
             //event->acceptProposedAction();     
             //event->ignore();                     // TODO CHECK Tim.
             }
@@ -485,7 +601,7 @@ void EventCanvas::endMoveItems(const QPoint& pos, DragType dragtype, int dir, bo
         MusEGlobal::song->applyOperationGroup(operations);
       
       moving.clear();
-      updateSelection();
+      itemSelectionsChanged();
       redraw();
       }
 
@@ -495,24 +611,40 @@ void EventCanvas::endMoveItems(const QPoint& pos, DragType dragtype, int dir, bo
 
 void EventCanvas::startPlayEvent(int note, int velocity, int port, int channel)
       {
+      // REMOVE Tim. Noteoff. Added. Zero note on vel is not allowed now.
+      if(velocity == 0)
+      {
+        fprintf(stderr, "EventCanvas::startPlayEvent: Warning: Zero note on velocity!\n");
+        velocity = 1;
+      }
+      
       if (MusEGlobal::debugMsg)
-        printf("EventCanvas::startPlayEvent %d %d %d %d\n", note, velocity, port, channel);
+        fprintf(stderr, "EventCanvas::startPlayEvent %d %d %d %d\n", note, velocity, port, channel);
 
       // Release any current note.
       stopPlayEvent();
       
-      playedPitch        = note + track()->transposition;
-      playedVelocity     = velocity;
-      playedPitchPort    = port;
-      playedPitchChannel = channel;
+      if(!track())
+        return;
+      
+      int playedPitch        = note;
+      // Apply track transposition, but only for midi tracks, not drum tracks.
+      if(track()->isMidiTrack() && !track()->isDrumTrack())
+        playedPitch += track()->transposition;
       
       // play note:
-      MusECore::MidiPlayEvent e(0, port, channel, MusECore::ME_NOTEON, playedPitch, velocity);
-      MusEGlobal::audio->msgPlayMidiEvent(&e);
+      if(stuckNoteExists(port, channel, playedPitch))
+        return;
+      const MusECore::MidiPlayEvent e(MusEGlobal::audio->curFrame(), port, channel, MusECore::ME_NOTEON, playedPitch, velocity);
+      _stuckNotes.push_back(e);
+      // Send to the port and device.
+      MusEGlobal::midiPorts[port].putEvent(e);
       }
 
 void EventCanvas::startPlayEvent(int note, int velocity)
       {
+      if(!track())
+        return;
       int port         = track()->outPort();
       int channel      = track()->outChannel();
       startPlayEvent(note, velocity, port, channel);
@@ -524,14 +656,26 @@ void EventCanvas::startPlayEvent(int note, int velocity)
 
 void EventCanvas::stopPlayEvent()
       {
-      if(playedPitch == -1 || playedPitchPort == -1 || playedPitchChannel == -1)
-        return;
-      // release note:
-      //MusECore::MidiPlayEvent ev(0, playedPitchPort, playedPitchChannel, 0x90, playedPitch, 0);  // REMOVE Tim.
-      MusECore::MidiPlayEvent ev(0, playedPitchPort, playedPitchChannel, MusECore::ME_NOTEOFF, playedPitch, playedVelocity);
-      MusEGlobal::audio->msgPlayMidiEvent(&ev);
-      playedPitch = playedPitchPort = playedPitchChannel = -1;
-      playedVelocity = 0;
+      // Stop all currently playing notes.
+      unsigned int frame = MusEGlobal::audio->curFrame();
+      int port;
+      const int sz = _stuckNotes.size();
+      for(int i = 0; i < sz; ++i)
+      {
+        MusECore::MidiPlayEvent ev(_stuckNotes.at(i));
+        port = ev.port();
+        if(port < 0 || port >= MusECore::MIDI_PORTS)
+          continue;
+        ev.setType(MusECore::ME_NOTEOFF);
+        ev.setTime(frame);
+        if(ev.dataB() == 0)
+          ev.setB(64);
+        
+        // Send to the port and device.
+        MusEGlobal::midiPorts[port].putEvent(ev);
+      }
+      // Clear the stuck notes list.
+      _stuckNotes.clear();
       }
 
 } // namespace MusEGui
