@@ -96,7 +96,7 @@ class AudioComponentRack : public ComponentRack
 
   public slots:
     virtual void configChanged();
-    virtual void songChanged(MusECore::SongChangedFlags_t);
+    virtual void songChanged(MusECore::SongChangedStruct_t);
     
   public:
     AudioComponentRack(MusECore::AudioTrack* track, int id = -1, bool _manageAuxs = false, QWidget* parent = 0, Qt::WindowFlags f = 0);
@@ -125,7 +125,7 @@ class AudioStrip : public Strip {
       
   private:
       GridPosStruct _preScrollAreaPos_A;
-      GridPosStruct _preScrollAreaPos_B;
+      //GridPosStruct _preScrollAreaPos_B;
       GridPosStruct _effectRackPos;
       GridPosStruct _stereoToolPos;
       GridPosStruct _preToolPos;
@@ -136,16 +136,16 @@ class AudioStrip : public Strip {
       GridPosStruct _sliderPos;
       GridPosStruct _sliderLabelPos;
       GridPosStruct _postScrollAreaPos_A;
-      GridPosStruct _postScrollAreaPos_B;
+      //GridPosStruct _postScrollAreaPos_B;
       GridPosStruct _offPos;
       GridPosStruct _recPos;
       GridPosStruct _mutePos;
       GridPosStruct _soloPos;
-      GridPosStruct _routesPos;
+      //GridPosStruct _routesPos;
       GridPosStruct _inRoutesPos;
       GridPosStruct _outRoutesPos;
       GridPosStruct _automationPos;
-      GridPosStruct _rightSpacerPos;
+      //GridPosStruct _rightSpacerPos;
       GridPosStruct _offMonRecPos;
     
       int channel;
@@ -170,7 +170,7 @@ class AudioStrip : public Strip {
       double volume;
       bool _volPressed;
 
-      ClipperLabel* _clipperLabel[MAX_CHANNELS];
+      ClipperLabel* _clipperLabel[MusECore::MAX_CHANNELS];
       QHBoxLayout* _clipperLayout;
 
       void setClipperTooltip(int ch);
@@ -200,7 +200,7 @@ class AudioStrip : public Strip {
 
    public slots:
       virtual void configChanged();
-      virtual void songChanged(MusECore::SongChangedFlags_t);
+      virtual void songChanged(MusECore::SongChangedStruct_t);
       void incVolume(int v);
       void incPan(int v);
 
