@@ -37,8 +37,15 @@
 #include <QWheelEvent>
 #include <QResizeEvent>
 #include <QTimer>
+#include <QList>
+#include <QRect>
+#include <QMenu>
+#include <QPainter>
+#include <QPoint>
 
-class QRect;
+#include <map>
+#include "muse_time.h"
+#include "time_stretch.h"
 
 namespace MusECore {
 class SndFileR;
@@ -62,11 +69,67 @@ namespace MusEGui {
 //    ''visual'' Wave Event
 //---------------------------------------------------------
 
-class WEvent : public CItem {
+class WEvent : public EItem {
    public:
-      WEvent(MusECore::Event& e, MusECore::Part* p, int height);
+      WEvent(const MusECore::Event& e, MusECore::Part* p, int height);
       };
 
+      
+struct StretchSelectedItem
+{
+  MusECore::StretchListItem::StretchEventType _type;
+  MusECore::SndFileR _sndFile;
+  
+  StretchSelectedItem(MusECore::StretchListItem::StretchEventType type, 
+                      MusECore::SndFileR sndFile = MusECore::SndFileR())
+  {
+    _type = type;
+    _sndFile = sndFile;
+  }
+};
+
+typedef std::multimap<MusECore::MuseFrame_t, StretchSelectedItem, std::less<MusECore::MuseFrame_t> > StretchSelectedList_t;
+typedef StretchSelectedList_t::iterator iStretchSelectedItem;
+typedef StretchSelectedList_t::const_iterator ciStretchSelectedItem;
+typedef StretchSelectedList_t::reverse_iterator riStretchSelectedItem;
+typedef StretchSelectedList_t::const_reverse_iterator criStretchSelectedItem;
+
+typedef std::pair<iStretchSelectedItem, iStretchSelectedItem> iStretchSelectedItemPair;
+typedef std::pair<ciStretchSelectedItem, ciStretchSelectedItem> ciStretchSelectedItemPair;
+
+typedef std::pair<MusECore::MuseFrame_t, StretchSelectedItem> StretchSelectedItemInsertPair_t;
+
+enum StretchControllerVals { stretchDoNothing, stretchStartMove, stretchMovingController, stretchAddNewController };
+struct StretchAutomationObject {
+  StretchSelectedList_t _stretchSelectedList;
+  StretchControllerVals _controllerState;
+  bool _moveController;
+  QPoint _startMovePoint;
+  
+  
+  
+//   QPoint startMovePoint;
+//   QList<MusECore::MuseFrame_t> currentCtrlFrameList;
+//   bool currentCtrlValid;
+//   //MusECore::CtrlList *currentCtrlList;
+//   MusECore::StretchList* currentCtrlList;
+//   MusECore::Track *currentTrack;
+//   bool moveController;
+//   StretchControllerVals controllerState;
+//   QString currentText;
+//   bool breakUndoCombo;
+  //QRect currentTextRect;
+  //QRect currentVertexRect;
+  //int currentTick;
+  //int currentYNorm;
+  
+  StretchAutomationObject()
+  {
+    _controllerState = stretchDoNothing;
+    _moveController = false;
+  }
+};
+
 //---------------------------------------------------------
 //   WaveCanvas
 //---------------------------------------------------------
@@ -77,14 +140,20 @@ class WaveCanvas : public EventCanvas {
       enum { NORMAL, DRAG } mode;
       enum { MUTE = 0, NORMALIZE, FADE_IN, FADE_OUT, REVERSE, GAIN, EDIT_EXTERNAL, CUT, COPY, PASTE }; //!< Modify operations
       
+      static const int _stretchAutomationPointDetectDist;
+      static const int _stretchAutomationPointWidthUnsel;
+      static const int _stretchAutomationPointWidthSel;
+      
       int yScale;
       int button;
       unsigned startSample;
       unsigned endSample;
       int colorMode;
       int selectionStart, selectionStop, dragstartx;
-      int lastGainvalue; //!< Stores the last used gainvalue when specifiying gain value in the editgain dialog
+      int lastGainvalue; //!< Stores the last used gainvalue when specifying gain value in the editgain dialog
       QString copiedPart;
+
+      StretchAutomationObject _stretchAutomation;
       
       //bool getUniqueTmpfileName(QString& newFilename); //!< Generates unique filename for temporary SndFile
       MusECore::WaveSelectionList getSelection(unsigned startpos, unsigned stoppos);
@@ -99,38 +168,41 @@ class WaveCanvas : public EventCanvas {
       void editExternal(unsigned file_format, unsigned file_samplerate, unsigned channels, float** data, unsigned length);
       //void applyLadspa(unsigned channels, float** data, unsigned length); //!< Apply LADSPA plugin on selection
 
-      
+      void drawStretchAutomation(QPainter& p, const QRect& r, WEvent* wevent) const;
+      MusECore::iStretchListItem stretchListHitTest(int types, QPoint pt, WEvent* wevent);
+      void setStretchAutomationCursor(QPoint pt);
+
    protected:
       virtual QPoint raster(const QPoint&) const;
-      void drawTickRaster(QPainter& p, int x, int y, int w, int h, int raster);
-      void drawParts(QPainter&, const QRect&, bool do_cur_part);
+      void drawParts(QPainter&, bool /*do_cur_part*/, const QRect&, const QRegion& = QRegion());
   
-      virtual void draw(QPainter&, const QRect&);
       virtual void viewMouseDoubleClickEvent(QMouseEvent*);
       virtual void wheelEvent(QWheelEvent*);
       virtual bool mousePress(QMouseEvent*);
       virtual void mouseMove(QMouseEvent* event);
-      virtual void mouseRelease(const QPoint&);
-      virtual void drawItem(QPainter&, const CItem*, const QRect&);
-      void drawTopItem(QPainter &p, const QRect &rect);
-      virtual void drawMoving(QPainter&, const CItem*, const QRect&);
-      virtual MusECore::Undo moveCanvasItems(CItemList&, int, int, DragType, bool rasterize = true);
+      virtual void mouseRelease(QMouseEvent*);
+      virtual void drawItem(QPainter&, const CItem*, const QRect&, const QRegion& = QRegion());
+      void drawMarkers(QPainter& p, const QRect& mr, const QRegion& mrg = QRegion());
+      
+      void drawTopItem(QPainter& p, const QRect& rect, const QRegion& = QRegion());
+      virtual void drawMoving(QPainter&, const CItem*, const QRect&, const QRegion& = QRegion());
+      virtual MusECore::Undo moveCanvasItems(CItemMap&, int, int, DragType, bool rasterize = true);
       virtual bool moveItem(MusECore::Undo&, CItem*, const QPoint&, DragType, bool rasterize = true);
       virtual CItem* newItem(const QPoint&, int);
       virtual void resizeItem(CItem*, bool noSnap, bool);
       virtual void newItem(CItem*, bool noSnap);
       virtual bool deleteItem(CItem*);
-      virtual void startDrag(CItem* item, bool copymode);
+      virtual void startDrag(CItem* item, DragType);
       virtual void dragEnterEvent(QDragEnterEvent* event);
       virtual void dragMoveEvent(QDragMoveEvent*);
       virtual void dragLeaveEvent(QDragLeaveEvent*);
-      virtual CItem* addItem(MusECore::Part*, MusECore::Event&);
+      virtual CItem* addItem(MusECore::Part*, const MusECore::Event&);
 
       int y2pitch(int) const;
       int pitch2y(int) const;
       inline int y2height(int) const { return height(); }
       inline int yItemOffset() const { return 0; }
-      virtual void drawCanvas(QPainter&, const QRect&);
+      virtual void drawCanvas(QPainter&, const QRect&, const QRegion& = QRegion());
       virtual void itemPressed(const CItem*);
       virtual void itemReleased(const CItem*, const QPoint&);
       virtual void itemMoved(const CItem*, const QPoint&);
@@ -138,6 +210,9 @@ class WaveCanvas : public EventCanvas {
       virtual void resizeEvent(QResizeEvent*);
       void adjustWaveOffset(); 
       
+      virtual QMenu* genItemPopup(CItem*);
+      virtual void itemPopup(CItem*, int, const QPoint&);
+      
    private slots:
       void setPos(int idx, unsigned val, bool adjustScrollbar);
 
@@ -174,12 +249,14 @@ class WaveCanvas : public EventCanvas {
             redraw();
             }
       QString getCaption() const;
-      void songChanged(MusECore::SongChangedFlags_t);
+      void songChanged(MusECore::SongChangedStruct_t);
       void range(int* s, int* e) const { *s = startSample; *e = endSample; }
       void selectAtTick(unsigned int tick);
       void selectAtFrame(unsigned int frame);
       void modifySelected(NoteInfo::ValType type, int val, bool delta_mode = true);
       void keyPress(QKeyEvent*);
+      void keyRelease(QKeyEvent* event);
+      void updateItems();
       };
 
 } // namespace MusEGui
