@@ -28,15 +28,16 @@
 //
 //=========================================================
 
-#include <cmath>
 #include <list>
 
 #include <QMessageBox>
 
+#include "muse_math.h"
 #include "libsynti/mono.h"
 
 #define RESOLUTION   16384
-// Make sure this number is unique among all the MESS synths.
+// Make sure this number is unique among all the MESS synths (including ticksynth) and DSSI, VST, LV2 and other host synths.
+// 127 is reserved for special MusE system messages.
 #define S1_UNIQUE_ID      6
 
 
@@ -59,10 +60,7 @@ class S1 : public MessMono {
 
       virtual void note(int channel, int pitch, int velo);
       //virtual void processMessages();
-      virtual void process(float** buffer, int offset, int n);
-      //virtual bool hasGui() const { return true; }
-      //virtual bool guiVisible() const { return _showGui; }
-      //virtual void showGui(bool);
+      virtual void process(unsigned pos, float** buffer, int offset, int n);
       virtual bool hasNativeGui() const { return true; }
       virtual bool nativeGuiVisible() const { return _showGui; }
       virtual void showNativeGui(bool);
@@ -143,7 +141,7 @@ void S1::note(int /*channel*/, int pitch, int velo)
 //    synthesize n samples into buffer+offset
 //---------------------------------------------------------
 
-void S1::process(float** buffer, int offset, int n)
+void S1::process(unsigned /*pos*/, float** buffer, int offset, int n)
       {
       if (gate == 0)
             return;
@@ -227,10 +225,10 @@ int S1::getControllerInfo(int id, const char** name, int* ctrl, int* min, int* m
 class QWidget;
 
 
-static Mess* instantiate(int sr, QWidget*, QString* /* projectPathPtr */, const char*)
+static Mess* instantiate(unsigned long long /*parentWinId*/, const char* /*name*/, const MessConfig* config)
       {
       S1* s1 = new S1();
-      s1->setSampleRate(sr);
+      s1->setSampleRate(config->_sampleRate);
       return s1;
       }
 
