@@ -26,21 +26,29 @@
 // 02111-1301, USA or point your web browser to http://www.gnu.org.
 //=========================================================
 
-#include <cmath>
 #include <stdio.h>
 #include <list>
 
+#include "muse_math.h"
 #include "libsynti/mess.h"
-#include "muse/midi.h"
-#include "muse/midictrl.h"
+#include "muse/midi_consts.h"
+#include "muse/midictrl_consts.h"
 
 #include "common_defs.h"
 #include "vam.h"
 #include "vamgui.h"
 #include "libsynti/mono.h"
 
+std::string VAM_configPath;
+
 // Denormalise floats, only actually needed for PIII and very recent PowerPC
-#define DENORMALISE(fv) (((*(unsigned int*)&(fv))&0x7f800000)==0)?0.0f:(fv)
+//#define DENORMALISE(fv) (((*(unsigned int*)&(fv))&0x7f800000)==0)?0.0f:(fv)
+inline float DENORMALISE(float fv)
+{
+   char *c = reinterpret_cast<char *>(&fv);
+   unsigned int *ui = reinterpret_cast<unsigned int *>(c);
+   return ((((*ui)&0x7f800000)) == 0)?0.0f:(fv);
+}
 
 // A fast, truncating towards 0 modulo function. ANSI C doesn't define
 // which % will do, most truncate towards -inf
@@ -193,20 +201,16 @@ class VAM : public MessMono {
 
     public:
       virtual int getControllerInfo(int, const char**, int*, int*, int*, int*) const;
-      //virtual void getInitData(int* n, const unsigned char**p) const;
       virtual void getInitData(int* n, const unsigned char**p);
       // This is only a kludge required to support old songs' midistates. Do not use in any new synth.
       virtual int oldMidiStateHeader(const unsigned char** data) const;
-      //virtual bool guiVisible() const;
-      //virtual void showGui(bool);
-      //virtual bool hasGui() const { return true; }
       virtual bool nativeGuiVisible() const;
       virtual void showNativeGui(bool);
       virtual bool hasNativeGui() const { return true; }
       virtual void getNativeGeometry(int* x, int* y, int* w, int* h) const;
       virtual void setNativeGeometry(int x, int y, int w, int h);
       virtual void processMessages();
-      virtual void process(float**, int, int);
+      virtual void process(unsigned pos, float**, int, int);
       virtual void note(int channel, int pitch, int velo);
       virtual bool setController(int channel, int ctrl, int val);
       virtual bool sysex(int, const unsigned char*);
@@ -325,13 +329,13 @@ bool VAM::init(const char* name)
       {
       gui = new VAMGui;
       gui->setWindowTitle(QString(name));
-      gui->show();
+      gui->hide();
 
       if (useCount == 0) {
           int i;
           float tmp;
           for(i = 0; i < CB_AMP_SIZE; i++) {
-              cb2amp_tab[i] = pow(10.0, double(i) / -300.0);
+              cb2amp_tab[i] = exp10(double(i) / -300.0);
               //cb2amp_tab[i] = 1.0 - i/(float)CB_AMP_SIZE;
               }
           for(i = 0; i < LIN2EXP_SIZE; i++) {
@@ -340,9 +344,9 @@ bool VAM::init(const char* name)
               }
           int sr = sampleRate();
           /* Build up denormalised oscilator wavetables, these are sample_rate
-             long, costs more RAM to create them but makes freqency calcs much
-             cheaper, and means that interpolation isn't that neccesary, esp if
-             you use integer frequncies */
+             long, costs more RAM to create them but makes frequency calcs much
+             cheaper, and means that interpolation isn't that necessary, esp if
+             you use integer frequencies */
           
           float *tmp_tbl = new float[sr];
           const int lag = sr/50;
@@ -471,7 +475,7 @@ void VAM::processMessages()
 //   Called from host, ONLY if output path is connected.
 //---------------------------------------------------------
 
-void VAM::process(float** ports, int offset, int sampleCount)
+void VAM::process(unsigned /*pos*/, float** ports, int offset, int sampleCount)
       {
       /*
       //
@@ -1097,9 +1101,10 @@ void VAM::setNativeGeometry(int x, int y, int w, int h)
 
 class QWidget;
 
-static Mess* instantiate(int sr, QWidget*, QString*, const char* name)
+static Mess* instantiate(unsigned long long /*parentWinId*/, const char* name, const MessConfig* config)
       {
-      VAM* vam = new VAM(sr);
+      VAM_configPath = std::string(config->_configPath);
+      VAM* vam = new VAM(config->_sampleRate);
       if (vam->init(name)) {
             delete vam;
             return 0;
